#!/usr/bin/env python
# SPDX-License-Identifier: ISC
#
# Copyright (c) 2022 by VMware, Inc. ("VMware")
#                       Shreenidhi A R <rshreenidhi@vmware.com>
# Used Copyright (c) 2018 by Network Device Education Foundation, Inc. ("NetDEF")
# in this file.
#
"""
Following scenerios are covered.
1. When there is change in route-map policy associated with default-originate, changes does not reflect.
2. When route-map associated with default-originate is deleted, default route doesn't get withdrawn
3. Update message is not being sent when only route-map is removed from the default-originate config.
4. SNT counter gets incremented on change of every policy associated with default-originate
5. Route-map with multiple match clauses causes inconsistencies with default-originate.
6. BGP-Default originate behaviour with BGP attributes
"""
import os
import sys
import time
import pytest
from lib.topolog import logger

# pylint: disable=C0413
# Import topogen and topotest helpers
from lib.topogen import Topogen, get_topogen
from lib.topojson import build_config_from_json
from lib.topolog import logger

from lib.bgp import (
    verify_bgp_convergence,
    create_router_bgp,
    get_prefix_count_route,
    modify_as_number,
    verify_bgp_rib,
    get_dut_as_number,
    verify_rib_default_route,
    verify_fib_default_route,
)
from lib.common_config import (
    verify_fib_routes,
    step,
    required_linux_kernel_version,
    create_route_maps,
    interface_status,
    create_prefix_lists,
    get_frr_ipv6_linklocal,
    start_topology,
    write_test_header,
    verify_prefix_lists,
    check_address_types,
    write_test_footer,
    reset_config_on_routers,
    create_static_routes,
    check_router_status,
    delete_route_maps,
)

# Save the Current Working Directory to find configuration files.
CWD = os.path.dirname(os.path.realpath(__file__))
sys.path.append(os.path.join(CWD, "../"))
sys.path.append(os.path.join(CWD, "../lib/"))

pytestmark = [pytest.mark.bgpd, pytest.mark.staticd]

# Required to instantiate the topology builder class.

# pylint: disable=C0413
# Import topogen and topotest helpers

# Global variables
topo = None
NETWORK1_1 = {"ipv4": "198.51.1.1/32", "ipv6": "2001:DB8::1:1/128"}
DEFAULT_ROUTES = {"ipv4": "0.0.0.0/0", "ipv6": "0::0/0"}
NEXT_HOP_IP = {"ipv4": "Null0", "ipv6": "Null0"}


def setup_module(mod):
    """
    Sets up the pytest environment

    * `mod`: module name
    """

    # Required linux kernel version for this suite to run.
    result = required_linux_kernel_version("4.15")
    if result is not True:
        pytest.skip("Kernel requirements are not met")

    testsuite_run_time = time.asctime(time.localtime(time.time()))
    logger.info("Testsuite start time: {}".format(testsuite_run_time))
    logger.info("=" * 40)

    logger.info("Running setup_module to create topology")

    # This function initiates the topology build with Topogen...
    json_file = "{}/bgp_default_originate_topo1.json".format(CWD)
    tgen = Topogen(json_file, mod.__name__)
    global topo
    topo = tgen.json_topo
    # ... and here it calls Mininet initialization functions.

    # Starting topology, create tmp files which are loaded to routers
    #  to start daemons and then start routers
    start_topology(tgen)

    # Creating configuration from JSON
    build_config_from_json(tgen, topo)

    global ADDR_TYPES
    global BGP_CONVERGENCE
    global DEFAULT_ROUTES
    global DEFAULT_ROUTE_NXT_HOP_R1, DEFAULT_ROUTE_NXT_HOP_R3
    ADDR_TYPES = check_address_types()
    BGP_CONVERGENCE = verify_bgp_convergence(tgen, topo)
    assert BGP_CONVERGENCE is True, "setup_module :Failed \n Error: {}".format(
        BGP_CONVERGENCE
    )

    DEFAULT_ROUTES = {"ipv4": "0.0.0.0/0", "ipv6": "0::0/0"}

    interface = topo["routers"]["r1"]["links"]["r2"]["interface"]
    ipv6_link_local = get_frr_ipv6_linklocal(tgen, "r1", intf=interface)
    ipv4_nxt_hop = topo["routers"]["r1"]["links"]["r2"]["ipv4"].split("/")[0]
    ipv6_nxt_hop = topo["routers"]["r1"]["links"]["r2"]["ipv6"].split("/")[0]
    DEFAULT_ROUTE_NXT_HOP_R1 = {"ipv4": ipv4_nxt_hop, "ipv6": ipv6_link_local}

    interface = topo["routers"]["r3"]["links"]["r2"]["interface"]
    ipv6_link_local = get_frr_ipv6_linklocal(tgen, "r3", intf=interface)
    ipv4_nxt_hop = topo["routers"]["r3"]["links"]["r2"]["ipv4"].split("/")[0]
    ipv6_nxt_hop = topo["routers"]["r3"]["links"]["r2"]["ipv6"].split("/")[0]
    DEFAULT_ROUTE_NXT_HOP_R3 = {"ipv4": ipv4_nxt_hop, "ipv6": ipv6_link_local}

    logger.info("Running setup_module() done")


def teardown_module():
    """Teardown the pytest environment"""

    logger.info("Running teardown_module to delete topology")

    tgen = get_topogen()

    # Stop toplogy and Remove tmp files
    tgen.stop_topology()

    logger.info(
        "Testsuite end time: {}".format(time.asctime(time.localtime(time.time())))
    )
    logger.info("=" * 40)


#####################################################
#
#                      Testcases
#
#####################################################


def test_default_originate_delete_conditional_routemap(request):
    """
    "scenerio covered":
    1. When there is change in route-map policy associated with default-originate, changes does not reflect.
    2. When route-map associated with default-originate is deleted, default route doesn't get withdrawn
    3. Update message is not being sent when only route-map is removed from the default-originate config.
    4. SNT counter gets incremented on change of every policy associated with default-originate
    5. Route-map with multiple match clauses causes inconsistencies with default-originate.
    """
    tgen = get_topogen()
    global BGP_CONVERGENCE
    global topo
    tc_name = request.node.name
    write_test_header(tc_name)
    tgen = get_topogen()
    if tgen.routers_have_failure():
        check_router_status(tgen)
    reset_config_on_routers(tgen)

    if BGP_CONVERGENCE != True:
        pytest.skip("skipped because of BGP Convergence failure")

    step("Configure IPv4 and IPv6 , IBGP neighbor between R1 and R2")
    step("Configure IPv4 and IPv6 , EBGP neighbor between R1 and R0")
    input_dict = {
        "r0": {
            "bgp": {
                "local_as": 999,
            }
        },
        "r1": {
            "bgp": {
                "local_as": 1000,
            }
        },
        "r2": {
            "bgp": {
                "local_as": 1000,
            }
        },
        "r3": {
            "bgp": {
                "local_as": 2000,
            }
        },
        "r4": {
            "bgp": {
                "local_as": 3000,
            }
        },
    }
    result = modify_as_number(tgen, topo, input_dict)
    try:
        assert result is True
    except AssertionError:
        logger.info("Expected behaviour: {}".format(result))
        logger.info("BGP config is not created because of invalid ASNs")

    step("After changing the BGP remote as , Verify the BGP Convergence")
    BGP_CONVERGENCE = verify_bgp_convergence(tgen, topo)
    assert (
        BGP_CONVERGENCE is True
    ), "Complete convergence is expected after changing ASN ....! ERROR :Failed \n Error: {}".format(
        BGP_CONVERGENCE
    )

    step("Configure 1 IPv4 and 1 IPv6 Static route on R0 with next-hop as Null0")
    for addr_type in ADDR_TYPES:
        static_routes_input = {
            "r0": {
                "static_routes": [
                    {
                        "network": [NETWORK1_1[addr_type]],
                        "next_hop": NEXT_HOP_IP[addr_type],
                    }
                ]
            }
        }
        result = create_static_routes(tgen, static_routes_input)
        assert (
            result is True
        ), "Testcase {} : Failed to configure the static route  \n Error: {}".format(
            tc_name, result
        )

    step("verify IPv4 and IPv6 static route are configured and up on R0")
    for addr_type in ADDR_TYPES:
        static_routes_input = {
            "r0": {
                "static_routes": [
                    {
                        "network": [NETWORK1_1[addr_type]],
                        "next_hop": NEXT_HOP_IP[addr_type],
                    },
                ]
            }
        }
        result = verify_fib_routes(tgen, addr_type, "r0", static_routes_input)
        assert (
            result is True
        ), "Testcase {} : routes {}  not found in R0 FIB  \n Error: {}".format(
            tc_name, static_routes_input, result
        )

    step(
        "Configure  redistribute static on IPv4 and IPv6 address family on R0 for R0 to R1 neighbor "
    )
    redistribute_static = {
        "r0": {
            "bgp": {
                "address_family": {
                    "ipv4": {"unicast": {"redistribute": [{"redist_type": "static"}]}},
                    "ipv6": {"unicast": {"redistribute": [{"redist_type": "static"}]}},
                }
            }
        }
    }
    result = create_router_bgp(tgen, topo, redistribute_static)
    assert (
        result is True
    ), "Testcase {} : Failed to configure redistribute  configuration....! \n Error: {}".format(
        tc_name, result
    )

    step("verify IPv4 and IPv6 static route are received on R1")
    for addr_type in ADDR_TYPES:
        static_routes_input = {
            "r0": {
                "static_routes": [
                    {
                        "network": [NETWORK1_1[addr_type]],
                        "next_hop": NEXT_HOP_IP[addr_type],
                    },
                ]
            }
        }
        result = verify_fib_routes(tgen, addr_type, "r1", static_routes_input)
        assert (
            result is True
        ), "Testcase {} : Failed... Routes {}  expected in r1 FIB after configuring the redistribute config on R0 \n Error: {}".format(
            tc_name, static_routes_input, result
        )

        result = verify_bgp_rib(tgen, addr_type, "r1", static_routes_input)
        assert (
            result is True
        ), "Testcase {} : Failed... Routes {}  expected in r1 RIB after configuring the redistribute config on R0\n Error: {}".format(
            tc_name, static_routes_input, result
        )

    step(
        "Configure IPv4 prefix-list 'Pv4' and and IPv6 prefix-list 'Pv6' on R1 to match BGP route Sv41, IPv6 route Sv61 permit "
    )
    input_dict_3 = {
        "r1": {
            "prefix_lists": {
                "ipv4": {
                    "Pv4": [
                        {
                            "seqid": "1",
                            "network": NETWORK1_1["ipv4"],
                            "action": "permit",
                        },
                    ]
                },
                "ipv6": {
                    "Pv6": [
                        {
                            "seqid": "1",
                            "network": NETWORK1_1["ipv6"],
                            "action": "permit",
                        },
                    ]
                },
            }
        }
    }
    result = create_prefix_lists(tgen, input_dict_3)
    assert (
        result is True
    ), "Testcase {} : Failed  to configure the prefix list \n Error: {}".format(
        tc_name, result
    )

    step(
        "Configure IPV4 and IPv6 route-map (RMv4 and RMv6) matching prefix-list (Pv4 and Pv6) respectively on R1"
    )
    input_dict_3 = {
        "r1": {
            "route_maps": {
                "RMv4": [
                    {
                        "action": "permit",
                        "seq_id": "1",
                        "match": {"ipv4": {"prefix_lists": "Pv4"}},
                        "set": {
                            "path": {
                                "as_num": "5555",
                                "as_action": "prepend",
                            }
                        },
                    },
                ],
                "RMv6": [
                    {
                        "action": "permit",
                        "seq_id": "1",
                        "match": {"ipv6": {"prefix_lists": "Pv6"}},
                        "set": {
                            "path": {
                                "as_num": "5555",
                                "as_action": "prepend",
                            }
                        },
                    },
                ],
            }
        }
    }
    result = create_route_maps(tgen, input_dict_3)
    assert (
        result is True
    ), "Testcase {} : Failed to configure the route map  \n Error: {}".format(
        tc_name, result
    )

    step(
        "Configure default-originate with route-map (RMv4 and RMv6) on R1, on BGP IPv4 and IPv6 address family "
    )
    local_as = get_dut_as_number(tgen, dut="r1")
    default_originate_config = {
        "r1": {
            "bgp": {
                "local_as": local_as,
                "address_family": {
                    "ipv4": {
                        "unicast": {"default_originate": {"r2": {"route_map": "RMv4"}}}
                    },
                    "ipv6": {
                        "unicast": {"default_originate": {"r2": {"route_map": "RMv6"}}}
                    },
                },
            }
        }
    }
    result = create_router_bgp(tgen, topo, default_originate_config)
    assert (
        result is True
    ), "Testcase {} : Failed to configure the default originate \n Error: {}".format(
        tc_name, result
    )

    step(
        "After configuring default-originate command , verify default  routes are advertised on R2 "
    )
    DEFAULT_ROUTES = {"ipv4": "0.0.0.0/0", "ipv6": "0::0/0"}
    result = verify_rib_default_route(
        tgen,
        topo,
        dut="r2",
        routes=DEFAULT_ROUTES,
        expected_nexthop=DEFAULT_ROUTE_NXT_HOP_R1,
        metric=0,
        expected_aspath="5555",
    )
    assert (
        result is True
    ), "Testcase {} : Failed to configure the default originate \n Error: {}".format(
        tc_name, result
    )

    result = verify_fib_default_route(
        tgen,
        topo,
        dut="r2",
        routes=DEFAULT_ROUTES,
        expected_nexthop=DEFAULT_ROUTE_NXT_HOP_R1,
        expected=True,
    )
    assert (
        result is True
    ), "Testcase {} : Failed to configure the default originate \n Error: {}".format(
        tc_name, result
    )

    step("Changing the as-path  policy of the existing route-map")
    input_dict_3 = {
        "r1": {
            "route_maps": {
                "RMv4": [
                    {
                        "action": "permit",
                        "seq_id": "1",
                        "match": {"ipv4": {"prefix_lists": "Pv4"}},
                        "set": {
                            "path": {
                                "as_num": "6666",
                                "as_action": "prepend",
                            }
                        },
                    },
                ],
                "RMv6": [
                    {
                        "action": "permit",
                        "seq_id": "1",
                        "match": {"ipv6": {"prefix_lists": "Pv6"}},
                        "set": {
                            "path": {
                                "as_num": "6666",
                                "as_action": "prepend",
                            }
                        },
                    },
                ],
            }
        }
    }
    result = create_route_maps(tgen, input_dict_3)
    assert (
        result is True
    ), "Testcase {} : Failed to configure the route map  \n Error: {}".format(
        tc_name, result
    )

    step(
        "Verify prefix sent count on R1 towards R2 \n Send count shoud not be incremented on change of existing (AS-path) policy "
    )
    snapshot = get_prefix_count_route(
        tgen, topo, dut="r1", peer="r2", link="r1", sent=True, received=False
    )

    ipv4_prefix_count = False
    ipv6_prefix_count = False
    if snapshot["ipv4_count"] == 2:
        ipv4_prefix_count = True
    if snapshot["ipv6_count"] == 2:
        ipv6_prefix_count = True

    assert (
        ipv4_prefix_count is True
    ), "Testcase {} : Failed Error: Expected sent Prefix is 2 but obtained {} ".format(
        tc_name, ipv4_prefix_count
    )
    assert (
        ipv6_prefix_count is True
    ), "Testcase {} : Failed Error: Expected sent Prefix is 2 but obtained {} ".format(
        tc_name, ipv6_prefix_count
    )

    step(
        "After changing the as-path policy verify  the  new policy is advertised to router  R2"
    )
    DEFAULT_ROUTES = {"ipv4": "0.0.0.0/0", "ipv6": "0::0/0"}
    result = verify_rib_default_route(
        tgen,
        topo,
        dut="r2",
        routes=DEFAULT_ROUTES,
        expected_nexthop=DEFAULT_ROUTE_NXT_HOP_R1,
        metric=0,
        expected_aspath="6666",
    )
    assert (
        result is True
    ), "Testcase {} : Default route with expected attributes is not found in BGP RIB  \n Error: {}".format(
        tc_name, result
    )

    result = verify_fib_default_route(
        tgen,
        topo,
        dut="r2",
        routes=DEFAULT_ROUTES,
        expected_nexthop=DEFAULT_ROUTE_NXT_HOP_R1,
        expected=True,
    )
    assert (
        result is True
    ), "Testcase {} : Default route with expected attributes is not found in BGP FIB  \n Error: {}".format(
        tc_name, result
    )

    step("Remove the as-path policy from the route-map")
    input_dict_3 = {
        "r1": {
            "route_maps": {
                "RMv4": [
                    {
                        "action": "permit",
                        "seq_id": "1",
                        "match": {"ipv4": {"prefix_lists": "Pv4"}},
                        "set": {
                            "path": {
                                "as_num": "6666",
                                "as_action": "prepend",
                                "delete": True,
                            }
                        },
                    },
                ],
                "RMv6": [
                    {
                        "action": "permit",
                        "seq_id": "1",
                        "match": {"ipv6": {"prefix_lists": "Pv6"}},
                        "set": {
                            "path": {
                                "as_num": "6666",
                                "as_action": "prepend",
                                "delete": True,
                            }
                        },
                    },
                ],
            }
        }
    }
    result = create_route_maps(tgen, input_dict_3)
    assert (
        result is True
    ), "Testcase {} : Failed to configure the route map  \n Error: {}".format(
        tc_name, result
    )

    step(
        "After removing the  route policy (AS-Path) verify that as-path is removed in r2 "
    )
    DEFAULT_ROUTES = {"ipv4": "0.0.0.0/0", "ipv6": "0::0/0"}

    result = verify_rib_default_route(
        tgen,
        topo,
        dut="r2",
        routes=DEFAULT_ROUTES,
        expected_nexthop=DEFAULT_ROUTE_NXT_HOP_R1,
    )
    assert result is True, "Testcase {} : Failed ... !  \n Error: {}".format(
        tc_name, result
    )

    result = verify_fib_default_route(
        tgen,
        topo,
        dut="r2",
        routes=DEFAULT_ROUTES,
        expected_nexthop=DEFAULT_ROUTE_NXT_HOP_R1,
        expected=True,
    )
    assert result is True, "Testcase {} : Failed .... !\n Error: {}".format(
        tc_name, result
    )

    step("Delete the route-map ")

    delete_routemap = {"r1": {"route_maps": ["RMv4", "RMv6"]}}
    result = delete_route_maps(tgen, delete_routemap)
    assert (
        result is True
    ), "Testcase {} : Failed  to delete the route-map\n Error: {}".format(
        tc_name, result
    )

    step(
        "After deleting route-map , verify  the default route  in FIB and RIB are removed "
    )
    DEFAULT_ROUTES = {"ipv4": "0.0.0.0/0", "ipv6": "0::0/0"}
    result = verify_rib_default_route(
        tgen,
        topo,
        dut="r2",
        routes=DEFAULT_ROUTES,
        expected_nexthop=DEFAULT_ROUTE_NXT_HOP_R1,
        metric=0,
        expected=False,
    )
    assert (
        result is not True
    ), "Testcase {} : After removing the route-map the default-route is not removed from R2 RIB\n Error: {}".format(
        tc_name, result
    )

    result = verify_fib_default_route(
        tgen,
        topo,
        dut="r2",
        routes=DEFAULT_ROUTES,
        expected_nexthop=DEFAULT_ROUTE_NXT_HOP_R1,
        expected=False,
    )
    assert (
        result is not True
    ), "Testcase {} :  After removing the route-map the default-route is not removed from R2 FIB \n Error: {}".format(
        tc_name, result
    )

    step("Create route-map with with sequnce number 10 ")
    input_dict_3 = {
        "r1": {
            "route_maps": {
                "RMv4": [
                    {
                        "action": "permit",
                        "seq_id": "10",
                        "match": {"ipv4": {"prefix_lists": "Pv4"}},
                        "set": {
                            "path": {
                                "as_num": "9999",
                                "as_action": "prepend",
                            }
                        },
                    },
                ],
                "RMv6": [
                    {
                        "action": "permit",
                        "seq_id": "10",
                        "match": {"ipv6": {"prefix_lists": "Pv6"}},
                        "set": {
                            "path": {
                                "as_num": "9999",
                                "as_action": "prepend",
                            }
                        },
                    },
                ],
            }
        }
    }
    result = create_route_maps(tgen, input_dict_3)
    assert (
        result is True
    ), "Testcase {} : Failed to configure the route map  \n Error: {}".format(
        tc_name, result
    )

    step(
        "After Configuring the route-map the dut is expected to receive the  route policy (as-path) as 99999"
    )
    DEFAULT_ROUTES = {"ipv4": "0.0.0.0/0", "ipv6": "0::0/0"}
    result = verify_rib_default_route(
        tgen,
        topo,
        dut="r2",
        routes=DEFAULT_ROUTES,
        expected_nexthop=DEFAULT_ROUTE_NXT_HOP_R1,
        metric=0,
        expected_aspath="9999",
    )
    assert result is True, "Testcase {} : Failed...!   \n Error: {}".format(
        tc_name, result
    )

    result = verify_fib_default_route(
        tgen,
        topo,
        dut="r2",
        routes=DEFAULT_ROUTES,
        expected_nexthop=DEFAULT_ROUTE_NXT_HOP_R1,
        expected=True,
    )
    assert result is True, "Testcase {} : Failed ...!\n Error: {}".format(
        tc_name, result
    )

    step("Create another route-map with seq number less than the previous i. <10 ")
    input_dict_3 = {
        "r1": {
            "route_maps": {
                "RMv4": [
                    {
                        "action": "permit",
                        "seq_id": "5",
                        "match": {"ipv4": {"prefix_lists": "Pv4"}},
                        "set": {
                            "path": {
                                "as_num": "7777",
                                "as_action": "prepend",
                            }
                        },
                    },
                ],
                "RMv6": [
                    {
                        "action": "permit",
                        "seq_id": "5",
                        "match": {"ipv6": {"prefix_lists": "Pv6"}},
                        "set": {
                            "path": {
                                "as_num": "7777",
                                "as_action": "prepend",
                            }
                        },
                    },
                ],
            }
        }
    }
    result = create_route_maps(tgen, input_dict_3)
    assert (
        result is True
    ), "Testcase {} : Failed to configure the route map  \n Error: {}".format(
        tc_name, result
    )

    step(
        "On creating new route-map  the route-map with lower seq id should be considered "
    )
    DEFAULT_ROUTES = {"ipv4": "0.0.0.0/0", "ipv6": "0::0/0"}
    result = verify_rib_default_route(
        tgen,
        topo,
        dut="r2",
        routes=DEFAULT_ROUTES,
        expected_nexthop=DEFAULT_ROUTE_NXT_HOP_R1,
        metric=0,
        expected_aspath="7777",
    )
    assert (
        result is True
    ), "Testcase {} : Route-map with lowest prefix is not considered  \n Error: {}".format(
        tc_name, result
    )

    result = verify_fib_default_route(
        tgen,
        topo,
        dut="r2",
        routes=DEFAULT_ROUTES,
        expected_nexthop=DEFAULT_ROUTE_NXT_HOP_R1,
        expected=True,
    )
    assert (
        result is True
    ), "Testcase {} :  Route-map with lowest prefix is not considered   \n Error: {}".format(
        tc_name, result
    )

    write_test_footer(tc_name)


def test_verify_default_originate_after_BGP_attributes_p1(request):
    """
    "Verify different BGP attributes with default-originate route "
    """
    tgen = get_topogen()
    global BGP_CONVERGENCE
    global topo
    # test case name
    tc_name = request.node.name
    write_test_header(tc_name)
    # Don't run this test if we have any failure.
    if tgen.routers_have_failure():
        check_router_status(tgen)
    reset_config_on_routers(tgen)

    if BGP_CONVERGENCE != True:
        pytest.skip("skipped because of BGP Convergence failure")

    step("Configure IPv4 and IPv6 , EBGP neighbor between R3 and R2")
    step("Configure IPv4 and IPv6 IBGP neighbor between R3 and R4")
    r0_local_as = topo["routers"]["r0"]["bgp"]["local_as"]
    r1_local_as = topo["routers"]["r1"]["bgp"]["local_as"]
    r2_local_as = topo["routers"]["r2"]["bgp"]["local_as"]
    r3_local_as = topo["routers"]["r3"]["bgp"]["local_as"]
    r4_local_as = topo["routers"]["r4"]["bgp"]["local_as"]
    input_dict = {
        "r0": {
            "bgp": {
                "local_as": r0_local_as,
            }
        },
        "r1": {
            "bgp": {
                "local_as": r1_local_as,
            }
        },
        "r2": {
            "bgp": {
                "local_as": r2_local_as,
            }
        },
        "r3": {
            "bgp": {
                "local_as": 4000,
            }
        },
        "r4": {
            "bgp": {
                "local_as": 4000,
            }
        },
    }
    result = modify_as_number(tgen, topo, input_dict)

    try:
        assert result is True
    except AssertionError:
        logger.info("Expected behaviour: {}".format(result))
        logger.info("BGP config is not created because of invalid ASNs")
    step("After changing the BGP AS Path Verify the BGP Convergence")

    BGP_CONVERGENCE = verify_bgp_convergence(tgen, topo)
    assert BGP_CONVERGENCE is True, "setup_module :Failed \n Error: {}".format(
        BGP_CONVERGENCE
    )

    step(
        "Configure one IPv4 and one IPv6, Static route on R4 with next-hop as Null0 IPv4 route Sv41, IPv6 route Sv61 "
    )
    for addr_type in ADDR_TYPES:
        static_routes_input = {
            "r4": {
                "static_routes": [
                    {
                        "network": [NETWORK1_1[addr_type]],
                        "next_hop": NEXT_HOP_IP[addr_type],
                    }
                ]
            }
        }
        result = create_static_routes(tgen, static_routes_input)
        assert result is True, "Testcase {} : Failed \n Error: {}".format(
            tc_name, result
        )
    step("Verify IPv4 and IPv6 static routes configured on R4 in FIB")
    for addr_type in ADDR_TYPES:
        static_routes_input = {
            "r4": {
                "static_routes": [
                    {
                        "network": [NETWORK1_1[addr_type]],
                        "next_hop": NEXT_HOP_IP[addr_type],
                    }
                ]
            }
        }
        result = verify_fib_routes(tgen, addr_type, "r4", static_routes_input)
        assert result is True, "Testcase {} : Failed \n Error: {}".format(
            tc_name, result
        )

    step(
        "Configure redistribute static knob on R4 , for R4 to R3 neighbor for IPv4 and IPv6 address family "
    )
    redistribute_static = {
        "r4": {
            "bgp": {
                "address_family": {
                    "ipv4": {"unicast": {"redistribute": [{"redist_type": "static"}]}},
                    "ipv6": {"unicast": {"redistribute": [{"redist_type": "static"}]}},
                }
            }
        }
    }
    result = create_router_bgp(tgen, topo, redistribute_static)
    assert result is True, "Testcase {} : Failed \n Error: {}".format(tc_name, result)

    step(
        "Verify After configuring redistribute static , verify route received in BGP table of R3"
    )
    for addr_type in ADDR_TYPES:
        static_routes_input = {
            "r3": {
                "static_routes": [
                    {
                        "network": [NETWORK1_1[addr_type]],
                        "next_hop": NEXT_HOP_IP[addr_type],
                    }
                ]
            }
        }
        result = verify_fib_routes(tgen, addr_type, "r3", static_routes_input)
        assert result is True, "Testcase {} : Failed \n Error: {}".format(
            tc_name, result
        )

        result = verify_bgp_rib(tgen, addr_type, "r3", static_routes_input)
        assert result is True, "Testcase {} : Failed \n Error: {}".format(
            tc_name, result
        )

    NOTE = """Configure 2  IPv4 prefix-list Pv41 Pv42 and and 2 IPv6 prefix-list Pv61 Pv62 on R3 to match BGP IPv4 route Sv41, 200.1.1.1/24 , IPv6 route Sv61 and 200::1/64"""
    step(NOTE)
    input_dict_3 = {
        "r3": {
            "prefix_lists": {
                "ipv4": {
                    "Pv41": [
                        {
                            "seqid": "1",
                            "network": NETWORK1_1["ipv4"],
                            "action": "permit",
                        }
                    ],
                    "Pv42": [
                        {"seqid": "1", "network": "200.1.1.1/24", "action": "permit"}
                    ],
                },
                "ipv6": {
                    "Pv61": [
                        {
                            "seqid": "1",
                            "network": NETWORK1_1["ipv6"],
                            "action": "permit",
                        }
                    ],
                    "Pv62": [
                        {"seqid": " 1", "network": "200::1/64", "action": "permit"}
                    ],
                },
            }
        }
    }
    result = create_prefix_lists(tgen, input_dict_3)
    assert result is True, "Testcase {} : Failed \n Error: {}".format(tc_name, result)

    step("verify IPv4 and IPv6 Prefix list got configured on R3")
    input_dict = {"r3": {"prefix_lists": ["Pv41", "Pv61", "Pv42", "Pv62"]}}
    result = verify_prefix_lists(tgen, input_dict)
    assert result is True, "Testcase {} : Failed \n Error: {}".format(tc_name, result)

    step(
        "Configure 2 sequence of route-map for IPv4 seq1 permit Pv41 and seq2 permit Pv42 and for IPv6 seq1 permit Pv61 , seq2 permit Pv62 on R3"
    )
    input_dict_3 = {
        "r3": {
            "route_maps": {
                "RMv4": [
                    {
                        "action": "permit",
                        "seq_id": "1",
                        "match": {"ipv4": {"prefix_lists": "Pv41"}},
                    },
                    {
                        "action": "permit",
                        "seq_id": "2",
                        "match": {"ipv4": {"prefix_lists": "Pv42"}},
                    },
                ],
                "RMv6": [
                    {
                        "action": "permit",
                        "seq_id": "1",
                        "match": {"ipv6": {"prefix_lists": "Pv61"}},
                    },
                    {
                        "action": "permit",
                        "seq_id": "2",
                        "match": {"ipv6": {"prefix_lists": "Pv62"}},
                    },
                ],
            }
        }
    }
    result = create_route_maps(tgen, input_dict_3)
    assert result is True, "Testcase {} : Failed \n Error: {}".format(tc_name, result)

    step(
        "Apply on route-map seq1 set as-path prepend to 200 and route-map seq2 set  as-path prepend to 300 for IPv4 and IPv6 route-map "
    )
    route_map = {
        "r3": {
            "route_maps": {
                "RMv4": [
                    {
                        "action": "permit",
                        "seq_id": "1",
                        "set": {
                            "path": {
                                "as_num": "200",
                                "as_action": "prepend",
                            }
                        },
                    },
                    {
                        "action": "permit",
                        "seq_id": "2",
                        "set": {
                            "path": {
                                "as_num": "300",
                                "as_action": "prepend",
                            }
                        },
                    },
                ],
                "RMv6": [
                    {
                        "action": "permit",
                        "seq_id": "1",
                        "set": {
                            "path": {
                                "as_num": "200",
                                "as_action": "prepend",
                            }
                        },
                    },
                    {
                        "action": "permit",
                        "seq_id": "2",
                        "set": {
                            "path": {
                                "as_num": "300",
                                "as_action": "prepend",
                            }
                        },
                    },
                ],
            }
        }
    }

    result = create_route_maps(tgen, route_map)
    assert result is True, "Test case {} : Failed \n Error: {}".format(tc_name, result)

    step(
        " Configure default-originate with IPv4 and IPv6 route-map on R3 for R3-R2 IPv4 and IPv6 BGP neighbor"
    )

    local_as = get_dut_as_number(tgen, dut="r3")
    default_originate_config = {
        "r3": {
            "bgp": {
                "local_as": local_as,
                "address_family": {
                    "ipv4": {
                        "unicast": {"default_originate": {"r2": {"route_map": "RMv4"}}}
                    },
                    "ipv6": {
                        "unicast": {"default_originate": {"r2": {"route_map": "RMv6"}}}
                    },
                },
            }
        }
    }
    result = create_router_bgp(tgen, topo, default_originate_config)
    assert result is True, "Testcase {} : Failed \n Error: {}".format(tc_name, result)

    step(
        "Verify IPv4 and IPv6 default route received on R2 with both the AS path on R2"
    )
    DEFAULT_ROUTES = {"ipv4": "0.0.0.0/0", "ipv6": "0::0/0"}

    result = verify_rib_default_route(
        tgen,
        topo,
        dut="r2",
        routes=DEFAULT_ROUTES,
        expected_nexthop=DEFAULT_ROUTE_NXT_HOP_R3,
        metric=0,
        expected_aspath="4000 200",
    )

    step(
        "Modify AS prepend path adding one more value 500 in route-map sequence 1 and 600 for route-map sequence 2 for IPv4 and IPv6 route-map"
    )
    route_map = {
        "r3": {
            "route_maps": {
                "RMv4": [
                    {
                        "action": "permit",
                        "seq_id": "1",
                        "set": {
                            "path": {
                                "as_num": "500",
                                "as_action": "prepend",
                            }
                        },
                    },
                    {
                        "action": "permit",
                        "seq_id": "2",
                        "set": {
                            "path": {
                                "as_num": "600",
                                "as_action": "prepend",
                            }
                        },
                    },
                ],
                "RMv6": [
                    {
                        "action": "permit",
                        "seq_id": "1",
                        "set": {
                            "path": {
                                "as_num": "500",
                                "as_action": "prepend",
                            }
                        },
                    },
                    {
                        "action": "permit",
                        "seq_id": "2",
                        "set": {
                            "path": {
                                "as_num": "600",
                                "as_action": "prepend",
                            }
                        },
                    },
                ],
            }
        }
    }

    result = create_route_maps(tgen, route_map)
    assert result is True, "Test case {} : Failed \n Error: {}".format(tc_name, result)
    assert result is True, "Testcase {} : Failed Error: {}".format(tc_name, result)

    step("As path 500 added to IPv4 and IPv6 default -originate route received on R2")
    result = verify_rib_default_route(
        tgen,
        topo,
        dut="r2",
        routes=DEFAULT_ROUTES,
        expected_nexthop=DEFAULT_ROUTE_NXT_HOP_R3,
        metric=0,
        expected_aspath="4000 500",
    )
    assert result is True, "Testcase {} : Failed Error: {}".format(tc_name, result)

    step(
        "Apply on route-map seq1 set metric value to  70 and route-map seq2 set  metric 80 IPv4 and IPv6 route-map"
    )
    route_map = {
        "r3": {
            "route_maps": {
                "RMv4": [
                    {
                        "action": "permit",
                        "seq_id": "1",
                        "set": {
                            "metric": 70,
                        },
                    },
                    {
                        "action": "permit",
                        "seq_id": "2",
                        "set": {
                            "metric": 80,
                        },
                    },
                ],
                "RMv6": [
                    {
                        "action": "permit",
                        "seq_id": "1",
                        "set": {
                            "metric": 70,
                        },
                    },
                    {
                        "action": "permit",
                        "seq_id": "2",
                        "set": {
                            "metric": 80,
                        },
                    },
                ],
            }
        }
    }

    result = create_route_maps(tgen, route_map)
    assert result is True, "Test case {} : Failed \n Error: {}".format(tc_name, result)

    step(
        "Verify Configured metric value received on R2 along with as-path for IPv4 and IPv6 default routes "
    )

    DEFAULT_ROUTES = {"ipv4": "0.0.0.0/0", "ipv6": "::/0"}
    result = verify_rib_default_route(
        tgen,
        topo,
        dut="r2",
        routes=DEFAULT_ROUTES,
        expected_nexthop=DEFAULT_ROUTE_NXT_HOP_R3,
        metric=70,
        expected_aspath="4000 500",
    )

    step(
        "Modify route-map seq1 configure metric 50 and route-map seq2 configure metric 100 IPv4 and IPv6 route-map "
    )
    route_map = {
        "r3": {
            "route_maps": {
                "RMv4": [
                    {
                        "action": "permit",
                        "seq_id": "1",
                        "set": {
                            "metric": 50,
                        },
                    },
                    {
                        "action": "permit",
                        "seq_id": "2",
                        "set": {
                            "metric": 100,
                        },
                    },
                ],
                "RMv6": [
                    {
                        "action": "permit",
                        "seq_id": "1",
                        "set": {
                            "metric": 50,
                        },
                    },
                    {
                        "action": "permit",
                        "seq_id": "2",
                        "set": {
                            "metric": 100,
                        },
                    },
                ],
            }
        }
    }

    result = create_route_maps(tgen, route_map)
    assert result is True, "Test case {} : Failed \n Error: {}".format(tc_name, result)

    step(
        "Verify Configured metric value received on R2 along with as-path for IPv4 and IPv6 default routes "
    )

    result = verify_rib_default_route(
        tgen,
        topo,
        dut="r2",
        routes=DEFAULT_ROUTES,
        expected_nexthop=DEFAULT_ROUTE_NXT_HOP_R3,
        metric=50,
        expected_aspath="4000 500",
    )
    assert result is True, "Testcase {} : Failed Error: {}".format(tc_name, result)

    step("Delete AS-prepend  from IP4 and IPv6 route-map configured on R3 ")
    route_map = {
        "r3": {
            "route_maps": {
                "RMv4": [
                    {
                        "action": "permit",
                        "seq_id": "1",
                        "set": {
                            "path": {
                                "as_num": "500",
                                "as_action": "prepend",
                                "delete": True,
                            },
                            "delete": True,
                        },
                    },
                    {
                        "action": "permit",
                        "seq_id": "2",
                        "set": {
                            "path": {
                                "as_num": "600",
                                "as_action": "prepend",
                                "delete": True,
                            },
                            "delete": True,
                        },
                    },
                ],
                "RMv6": [
                    {
                        "action": "permit",
                        "seq_id": "1",
                        "set": {
                            "path": {
                                "as_num": "500",
                                "as_action": "prepend",
                                "delete": True,
                            },
                            "delete": True,
                        },
                    },
                    {
                        "action": "permit",
                        "seq_id": "2",
                        "set": {
                            "path": {
                                "as_num": "600",
                                "as_action": "prepend",
                                "delete": True,
                            },
                            "delete": True,
                        },
                    },
                ],
            }
        }
    }

    result = create_route_maps(tgen, route_map)
    assert result is True, "Test case {} : Failed \n Error: {}".format(tc_name, result)

    step(
        "Verify AS-prepend is deleted from default originate route and metric value only present on R2 for IPv4 and IPv6 default routes "
    )

    result = verify_rib_default_route(
        tgen,
        topo,
        dut="r2",
        routes=DEFAULT_ROUTES,
        expected_nexthop=DEFAULT_ROUTE_NXT_HOP_R3,
        metric=50,
        expected_aspath="4000",
    )
    assert result is True, "Testcase {} : Failed Error: {}".format(tc_name, result)

    step("Delete metric value  from IP4 and IPv6 route-map configured on R3 ")
    route_map = {
        "r3": {
            "route_maps": {
                "RMv4": [
                    {
                        "action": "permit",
                        "seq_id": "1",
                        "set": {"metric": 50, "delete": True},
                    },
                    {
                        "action": "permit",
                        "seq_id": "2",
                        "set": {"metric": 100, "delete": True},
                    },
                ],
                "RMv6": [
                    {
                        "action": "permit",
                        "seq_id": "1",
                        "set": {"metric": 50, "delete": True},
                    },
                    {
                        "action": "permit",
                        "seq_id": "2",
                        "set": {"metric": 100, "delete": True},
                    },
                ],
            }
        }
    }

    result = create_route_maps(tgen, route_map)
    assert result is True, "Test case {} : Failed \n Error: {}".format(tc_name, result)

    step(
        "Verify Metric value deleted from IPv4 and IPv6 default route on R2 ,verify  default routes "
    )

    result = verify_rib_default_route(
        tgen,
        topo,
        dut="r2",
        routes=DEFAULT_ROUTES,
        expected_nexthop=DEFAULT_ROUTE_NXT_HOP_R3,
        metric=0,
        expected_aspath="4000",
    )

    assert result is True, "Testcase {} : Failed Error: {}".format(tc_name, result)
    step("Change  IPv4 and IPv6 , EBGP to IBGP neighbor between R3 and R2")
    step("Change IPv4 and IPv6 IBGP to EBGP neighbor between R3 and R4")
    r0_local_as = topo["routers"]["r0"]["bgp"]["local_as"]
    r1_local_as = topo["routers"]["r1"]["bgp"]["local_as"]
    r2_local_as = topo["routers"]["r2"]["bgp"]["local_as"]
    r3_local_as = topo["routers"]["r3"]["bgp"]["local_as"]
    r4_local_as = topo["routers"]["r4"]["bgp"]["local_as"]
    input_dict = {
        "r0": {
            "bgp": {
                "local_as": r0_local_as,
            }
        },
        "r1": {
            "bgp": {
                "local_as": r1_local_as,
            }
        },
        "r2": {
            "bgp": {
                "local_as": 1111,
            }
        },
        "r3": {
            "bgp": {
                "local_as": 1111,
            }
        },
        "r4": {
            "bgp": {
                "local_as": 5555,
            }
        },
    }
    result = modify_as_number(tgen, topo, input_dict)
    try:
        assert result is True
    except AssertionError:
        logger.info("Expected behaviour: {}".format(result))
        logger.info("BGP config is not created because of invalid ASNs")
    step("After changing the BGP AS Path Verify the BGP Convergence")
    BGP_CONVERGENCE = verify_bgp_convergence(tgen, topo)
    assert BGP_CONVERGENCE is True, "setup_module :Failed \n Error: {}".format(
        BGP_CONVERGENCE
    )
    step(
        "Configure one IPv4 and one IPv6, Static route on R4 with next-hop as Null0 IPv4 route Sv41, IPv6 route Sv61 "
    )
    for addr_type in ADDR_TYPES:
        static_routes_input = {
            "r4": {
                "static_routes": [
                    {
                        "network": [NETWORK1_1[addr_type]],
                        "next_hop": NEXT_HOP_IP[addr_type],
                    }
                ]
            }
        }
        result = create_static_routes(tgen, static_routes_input)
        assert result is True, "Testcase {} : Failed \n Error: {}".format(
            tc_name, result
        )
    step("Verify IPv4 and IPv6 static routes configured on R4 in FIB")
    for addr_type in ADDR_TYPES:
        static_routes_input = {
            "r4": {
                "static_routes": [
                    {
                        "network": [NETWORK1_1[addr_type]],
                        "next_hop": NEXT_HOP_IP[addr_type],
                    }
                ]
            }
        }
        result = verify_fib_routes(tgen, addr_type, "r4", static_routes_input)
        assert result is True, "Testcase {} : Failed \n Error: {}".format(
            tc_name, result
        )

    step(
        "Configure redistribute static knob on R4 , for R4 to R3 neighbor for IPv4 and IPv6 address family "
    )
    redistribute_static = {
        "r4": {
            "bgp": {
                "address_family": {
                    "ipv4": {"unicast": {"redistribute": [{"redist_type": "static"}]}},
                    "ipv6": {"unicast": {"redistribute": [{"redist_type": "static"}]}},
                }
            }
        }
    }
    result = create_router_bgp(tgen, topo, redistribute_static)
    assert result is True, "Testcase {} : Failed \n Error: {}".format(tc_name, result)

    step(
        "Verify After configuring redistribute static , verify route received in BGP table of R3"
    )
    for addr_type in ADDR_TYPES:
        static_routes_input = {
            "r3": {
                "static_routes": [
                    {
                        "network": [NETWORK1_1[addr_type]],
                        "next_hop": NEXT_HOP_IP[addr_type],
                    }
                ]
            }
        }
        result = verify_fib_routes(tgen, addr_type, "r3", static_routes_input)
        assert result is True, "Testcase {} : Failed \n Error: {}".format(
            tc_name, result
        )

        result = verify_bgp_rib(tgen, addr_type, "r3", static_routes_input)
        assert result is True, "Testcase {} : Failed \n Error: {}".format(
            tc_name, result
        )
    step(
        " Configure default-originate with IPv4 and IPv6 route-map on R3 for R3-R2 IPv4 and IPv6 BGP neighbor"
    )
    local_as = get_dut_as_number(tgen, dut="r3")
    default_originate_config = {
        "r3": {
            "bgp": {
                "local_as": local_as,
                "address_family": {
                    "ipv4": {
                        "unicast": {"default_originate": {"r2": {"route_map": "RMv4"}}}
                    },
                    "ipv6": {
                        "unicast": {"default_originate": {"r2": {"route_map": "RMv6"}}}
                    },
                },
            }
        }
    }
    result = create_router_bgp(tgen, topo, default_originate_config)
    assert result is True, "Testcase {} : Failed \n Error: {}".format(tc_name, result)

    step(
        "Verify IPv4 and IPv6 default route received on R2 with both the AS path on R2"
    )
    DEFAULT_ROUTES = {"ipv4": "0.0.0.0/0", "ipv6": "0::0/0"}
    result = verify_rib_default_route(
        tgen,
        topo,
        dut="r2",
        routes=DEFAULT_ROUTES,
        expected_nexthop=DEFAULT_ROUTE_NXT_HOP_R3,
    )
    assert result is True, "Testcase {} : Failed Error: {}".format(tc_name, result)

    step(
        "Configure local -preference to 50 on IPv4 and IPv6 route map seq1 and 60 on seq2"
    )
    route_map = {
        "r3": {
            "route_maps": {
                "RMv4": [
                    {
                        "action": "permit",
                        "seq_id": "1",
                        "set": {
                            "locPrf": 50,
                        },
                    },
                    {
                        "action": "permit",
                        "seq_id": "2",
                        "set": {
                            "locPrf": 60,
                        },
                    },
                ],
                "RMv6": [
                    {
                        "action": "permit",
                        "seq_id": "1",
                        "set": {
                            "locPrf": 50,
                        },
                    },
                    {
                        "action": "permit",
                        "seq_id": "2",
                        "set": {
                            "locPrf": 60,
                        },
                    },
                ],
            }
        }
    }

    result = create_route_maps(tgen, route_map)
    assert result is True, "Test case {} : Failed \n Error: {}".format(tc_name, result)

    step(
        "Verify Configured metric value received on R2 along with as-path for IPv4 and IPv6 default routes "
    )

    result = verify_rib_default_route(
        tgen,
        topo,
        dut="r2",
        routes=DEFAULT_ROUTES,
        expected_nexthop=DEFAULT_ROUTE_NXT_HOP_R3,
        locPrf=50,
    )

    assert result is True, "Testcase {} : Failed Error: {}".format(tc_name, result)

    step(
        "Modify local preference value to 150  on IPv4 and IPv6 route map seq1 and 160 on seq2"
    )
    route_map = {
        "r3": {
            "route_maps": {
                "RMv4": [
                    {
                        "action": "permit",
                        "seq_id": "1",
                        "set": {
                            "locPrf": 150,
                        },
                    },
                    {
                        "action": "permit",
                        "seq_id": "2",
                        "set": {
                            "locPrf": 160,
                        },
                    },
                ],
                "RMv6": [
                    {
                        "action": "permit",
                        "seq_id": "1",
                        "set": {
                            "locPrf": 150,
                        },
                    },
                    {
                        "action": "permit",
                        "seq_id": "2",
                        "set": {
                            "locPrf": 160,
                        },
                    },
                ],
            }
        }
    }

    result = create_route_maps(tgen, route_map)
    assert result is True, "Test case {} : Failed \n Error: {}".format(tc_name, result)

    step(
        "Verify Modified local-preference  value received on R2  for IPv4 and IPv6 default routes "
    )

    DEFAULT_ROUTES = {"ipv4": "0.0.0.0/0", "ipv6": "::/0"}
    result = verify_rib_default_route(
        tgen,
        topo,
        dut="r2",
        routes=DEFAULT_ROUTES,
        expected_nexthop=DEFAULT_ROUTE_NXT_HOP_R3,
        locPrf=150,
    )

    assert result is True, "Testcase {} : Failed Error: {}".format(tc_name, result)
    # updating the topology with the updated AS-Number to avoid conflict in con configuring the AS
    updated_topo = topo
    updated_topo["routers"]["r0"]["bgp"]["local_as"] = get_dut_as_number(tgen, "r0")
    updated_topo["routers"]["r1"]["bgp"]["local_as"] = get_dut_as_number(tgen, "r1")
    updated_topo["routers"]["r2"]["bgp"]["local_as"] = get_dut_as_number(tgen, "r2")
    updated_topo["routers"]["r3"]["bgp"]["local_as"] = get_dut_as_number(tgen, "r3")
    updated_topo["routers"]["r4"]["bgp"]["local_as"] = get_dut_as_number(tgen, "r4")

    step(
        "Shut IPv4/IPv6 BGP neighbor from R4 ( R4-R3) using 'neighbor x.x.x.x shut' command "
    )
    local_as = get_dut_as_number(tgen, dut="r4")
    shut_neighbor = {
        "r4": {
            "bgp": {
                "local_as": local_as,
                "address_family": {
                    "ipv4": {
                        "unicast": {
                            "neighbor": {
                                "r3": {"dest_link": {"r4": {"shutdown": True}}}
                            }
                        }
                    },
                    "ipv6": {
                        "unicast": {
                            "neighbor": {
                                "r3": {"dest_link": {"r4": {"shutdown": True}}}
                            }
                        }
                    },
                },
            }
        }
    }
    result = create_router_bgp(tgen, updated_topo, shut_neighbor)
    assert result is True, "Testcase {} : Failed \n Error: {}".format(tc_name, result)

    interface = topo["routers"]["r3"]["links"]["r4"]["interface"]
    input_dict = {"r1": {"interface_list": [interface], "status": "down"}}

    result = interface_status(tgen, topo, input_dict)
    assert (
        result is True
    ), "Testcase {} : Shut down the interface failed ! \n Error: {}".format(
        tc_name, result
    )

    step("After shutting the interface verify the BGP convergence")
    result = verify_bgp_convergence(tgen, topo, expected=False)
    assert (
        result is not True
    ), "Testcase {} : Failed \n After shutting Down BGP convergence should Fail and return False \n Error: {}".format(
        tc_name, result
    )

    step("verify default route deleted from R2 ")
    result = verify_rib_default_route(
        tgen,
        topo,
        dut="r2",
        routes=DEFAULT_ROUTES,
        expected_nexthop=DEFAULT_ROUTE_NXT_HOP_R3,
        expected=False,
    )
    assert (
        result is not True
    ), "Testcase {} : Failed \n Error: After Shut down interface the default route is NOT expected but found in RIB -> {}".format(
        tc_name, result
    )

    result = verify_fib_default_route(
        tgen,
        topo,
        dut="r2",
        routes=DEFAULT_ROUTES,
        expected_nexthop=DEFAULT_ROUTE_NXT_HOP_R3,
        expected=False,
    )
    assert (
        result is not True
    ), "Testcase {} : Failed \n Error:  After Shut down interface the default route is NOT expected but found in FIB -> {}".format(
        tc_name, result
    )

    step(
        "no Shut IPv4/IPv6 BGP neighbor from R4 ( R4-R3) using 'neighbor x.x.x.x shut' command "
    )
    local_as = get_dut_as_number(tgen, dut="r4")
    shut_neighbor = {
        "r4": {
            "bgp": {
                "local_as": local_as,
                "address_family": {
                    "ipv4": {
                        "unicast": {
                            "neighbor": {
                                "r3": {"dest_link": {"r4": {"shutdown": False}}}
                            }
                        }
                    },
                    "ipv6": {
                        "unicast": {
                            "neighbor": {
                                "r3": {"dest_link": {"r4": {"shutdown": False}}}
                            }
                        }
                    },
                },
            }
        }
    }
    result = create_router_bgp(tgen, updated_topo, shut_neighbor)
    assert result is True, "Testcase {} : Failed \n Error: {}".format(tc_name, result)

    interface = topo["routers"]["r3"]["links"]["r4"]["interface"]
    input_dict = {"r1": {"interface_list": [interface], "status": "up"}}

    result = interface_status(tgen, topo, input_dict)
    assert (
        result is True
    ), "Testcase {} : Bring up interface failed ! \n Error: {}".format(tc_name, result)

    step("After no shutting the interface verify the BGP convergence")
    result = verify_bgp_convergence(tgen, topo, expected=True)
    assert (
        result is True
    ), "Testcase {} : Failed \n After shutting Down BGP convergence should Fail and return False \n Error: {}".format(
        tc_name, result
    )

    step("After no shut neighbor , verify default route relearn on R2")
    result = verify_rib_default_route(
        tgen,
        topo,
        dut="r2",
        routes=DEFAULT_ROUTES,
        expected_nexthop=DEFAULT_ROUTE_NXT_HOP_R3,
        expected=True,
    )
    assert (
        result is True
    ), "Testcase {} : Failed \n Error: After no Shut down interface the default route is  expected but found in RIB -> {}".format(
        tc_name, result
    )

    result = verify_fib_default_route(
        tgen,
        topo,
        dut="r2",
        routes=DEFAULT_ROUTES,
        expected_nexthop=DEFAULT_ROUTE_NXT_HOP_R3,
        expected=True,
    )
    assert (
        result is True
    ), "Testcase {} : Failed \n Error:  After Shut down interface the default route is  expected but found in FIB -> {}".format(
        tc_name, result
    )

    step("Remove IPv4/IPv6 static route configure on R4")
    for addr_type in ADDR_TYPES:
        static_routes_input = {
            "r4": {
                "static_routes": [
                    {
                        "network": [NETWORK1_1[addr_type]],
                        "next_hop": NEXT_HOP_IP[addr_type],
                        "delete": True,
                    }
                ]
            }
        }
        result = create_static_routes(tgen, static_routes_input)
        assert result is True, "Testcase {} : Failed \n Error: {}".format(
            tc_name, result
        )
    step("Verify IPv4 and IPv6 static routes removed on R4 in FIB")
    for addr_type in ADDR_TYPES:
        static_routes_input = {
            "r4": {
                "static_routes": [
                    {
                        "network": [NETWORK1_1[addr_type]],
                        "next_hop": NEXT_HOP_IP[addr_type],
                    }
                ]
            }
        }
        result = verify_fib_routes(
            tgen, addr_type, "r4", static_routes_input, expected=False
        )
        assert result is not True, "Testcase {} : Failed \n Error: {}".format(
            tc_name, result
        )
        result = verify_bgp_rib(
            tgen, addr_type, "r4", static_routes_input, expected=False
        )
        assert result is not True, "Testcase {} : Failed \n Error: {}".format(
            tc_name, result
        )

    step("After removing static route  , verify default route removed on R2")
    result = verify_rib_default_route(
        tgen,
        topo,
        dut="r2",
        routes=DEFAULT_ROUTES,
        expected_nexthop=DEFAULT_ROUTE_NXT_HOP_R3,
        expected=False,
    )
    assert (
        result is not True
    ), "Testcase {} : Failed \n Error: After removing static  the default route is NOT expected but found in RIB -> {}".format(
        tc_name, result
    )

    result = verify_fib_default_route(
        tgen,
        topo,
        dut="r2",
        routes=DEFAULT_ROUTES,
        expected_nexthop=DEFAULT_ROUTE_NXT_HOP_R3,
        expected=False,
    )
    assert (
        result is not True
    ), "Testcase {} : Failed \n Error:  After removing static the default route is NOT expected but found in FIB -> {}".format(
        tc_name, result
    )

    step("Configuring the static route back in r4")
    for addr_type in ADDR_TYPES:
        static_routes_input = {
            "r4": {
                "static_routes": [
                    {
                        "network": [NETWORK1_1[addr_type]],
                        "next_hop": NEXT_HOP_IP[addr_type],
                    }
                ]
            }
        }
        result = create_static_routes(tgen, static_routes_input)
        assert result is True, "Testcase {} : Failed \n Error: {}".format(
            tc_name, result
        )
    step("Verify IPv4 and IPv6 static routes configured on R4 in FIB")
    for addr_type in ADDR_TYPES:
        static_routes_input = {
            "r4": {
                "static_routes": [
                    {
                        "network": [NETWORK1_1[addr_type]],
                        "next_hop": NEXT_HOP_IP[addr_type],
                    }
                ]
            }
        }
        result = verify_fib_routes(
            tgen, addr_type, "r4", static_routes_input, expected=True
        )
        assert result is True, "Testcase {} : Failed \n Error: {}".format(
            tc_name, result
        )
        result = verify_bgp_rib(
            tgen, addr_type, "r4", static_routes_input, expected=True
        )
        assert result is True, "Testcase {} : Failed \n Error: {}".format(
            tc_name, result
        )

    step("After adding static route back  , verify default route learned  on R2")
    result = verify_rib_default_route(
        tgen,
        topo,
        dut="r2",
        routes=DEFAULT_ROUTES,
        expected_nexthop=DEFAULT_ROUTE_NXT_HOP_R3,
        expected=True,
    )
    assert (
        result is True
    ), "Testcase {} : Failed \n Error: After removing static  the default route is  expected but found in RIB -> {}".format(
        tc_name, result
    )

    result = verify_fib_default_route(
        tgen,
        topo,
        dut="r2",
        routes=DEFAULT_ROUTES,
        expected_nexthop=DEFAULT_ROUTE_NXT_HOP_R3,
        expected=True,
    )
    assert (
        result is True
    ), "Testcase {} : Failed \n Error:  After removing static  the default route is  expected but found in FIB -> {}".format(
        tc_name, result
    )

    step("Deactivate IPv4 and IPv6 neighbor configured from R4 ( R4-R3)")

    configure_bgp_on_r1 = {
        "r4": {
            "bgp": {
                "address_family": {
                    "ipv4": {
                        "unicast": {
                            "neighbor": {
                                "r3": {"dest_link": {"r4": {"deactivate": "ipv4"}}}
                            }
                        },
                    },
                    "ipv6": {
                        "unicast": {
                            "neighbor": {
                                "r3": {"dest_link": {"r4": {"deactivate": "ipv6"}}}
                            }
                        },
                    },
                }
            }
        }
    }
    result = create_router_bgp(tgen, updated_topo, configure_bgp_on_r1)
    assert result is True, "Testcase {} : Failed \n Error: {}".format(tc_name, result)

    step("After deactivating the BGP neighbor   , verify default route removed on R2")
    result = verify_rib_default_route(
        tgen,
        topo,
        dut="r2",
        routes=DEFAULT_ROUTES,
        expected_nexthop=DEFAULT_ROUTE_NXT_HOP_R3,
        expected=False,
    )
    assert (
        result is not True
    ), "Testcase {} : Failed \n Error: After Deactivating the BGP neighbor the default route is NOT expected but found in RIB -> {}".format(
        tc_name, result
    )

    result = verify_fib_default_route(
        tgen,
        topo,
        dut="r2",
        routes=DEFAULT_ROUTES,
        expected_nexthop=DEFAULT_ROUTE_NXT_HOP_R3,
        expected=False,
    )
    assert (
        result is not True
    ), "Testcase {} : Failed \n Error: After Deactivating the BGP neighbor  the default route is NOT expected but found in FIB -> {}".format(
        tc_name, result
    )

    step("Activate IPv4 and IPv6 neighbor configured from R4 ( R4-R3)")

    configure_bgp_on_r1 = {
        "r4": {
            "bgp": {
                "address_family": {
                    "ipv4": {
                        "unicast": {
                            "neighbor": {
                                "r3": {"dest_link": {"r4": {"activate": "ipv4"}}}
                            }
                        },
                    },
                    "ipv6": {
                        "unicast": {
                            "neighbor": {
                                "r3": {"dest_link": {"r4": {"activate": "ipv6"}}}
                            }
                        },
                    },
                }
            }
        }
    }
    result = create_router_bgp(tgen, updated_topo, configure_bgp_on_r1)
    assert result is True, "Testcase {} : Failed \n Error: {}".format(tc_name, result)

    step("Verify bgp convergence.")
    bgp_convergence = verify_bgp_convergence(tgen, updated_topo)
    assert bgp_convergence is True, "Testcase {} : Failed \n Error: {}".format(
        tc_name, bgp_convergence
    )
    step("After Activating the BGP neighbor   , verify default route learned on R2")
    result = verify_rib_default_route(
        tgen,
        topo,
        dut="r2",
        routes=DEFAULT_ROUTES,
        expected_nexthop=DEFAULT_ROUTE_NXT_HOP_R3,
        expected=True,
    )
    assert (
        result is True
    ), "Testcase {} : Failed \n Error: After Deactivating the BGP neighbor the default route is   expected but found in RIB -> {}".format(
        tc_name, result
    )

    result = verify_fib_default_route(
        tgen,
        topo,
        dut="r2",
        routes=DEFAULT_ROUTES,
        expected_nexthop=DEFAULT_ROUTE_NXT_HOP_R3,
        expected=True,
    )
    assert (
        result is True
    ), "Testcase {} : Failed \n Error:  After Deactivating the BGP neighbor the default route is   expected but found in FIB -> {}".format(
        tc_name, result
    )
    write_test_footer(tc_name)


if __name__ == "__main__":
    args = ["-s"] + sys.argv[1:]
    sys.exit(pytest.main(args))



#!/usr/bin/env python
# SPDX-License-Identifier: ISC

#
# test_ospf_unnumbered_point_to_multipoint.py
#
# Copyright (c) 2024 by
# Vincent Jardin
#

"""
test_ospf_unnumbered_point_to_multipoint.py: Test the OSPF unnumbered for routers with point to multipoint over Ethernet
"""

import os
import sys
from functools import partial
import pytest
import json

# Save the Current Working Directory to find configuration files.
CWD = os.path.dirname(os.path.realpath(__file__))
sys.path.append(os.path.join(CWD, "../"))

# pylint: disable=C0413
# Import topogen and topotest helpers
from lib import topotest
from lib.topogen import Topogen, TopoRouter, get_topogen
from lib.topolog import logger

# Required to instantiate the topology builder class.

pytestmark = [pytest.mark.ospfd]


CWD = os.path.dirname(os.path.realpath(__file__))


def build_topo(tgen):
    "Build function"

    # Create routers
    for routern in range(1, 4):
        tgen.add_router("r{}".format(routern))

    # Create a empty network for router 1
    switch = tgen.add_switch("s1")
    switch.add_link(tgen.gears["r1"])

    # Create a empty network for router 2
    switch = tgen.add_switch("s2")
    switch.add_link(tgen.gears["r2"])

    # Create a empty network for router 3
    switch = tgen.add_switch("s3")
    switch.add_link(tgen.gears["r3"])

    # Interconect router 1, 2 and r3 to a common switch 4
    switch = tgen.add_switch("s4")
    switch.add_link(tgen.gears["r1"])
    switch.add_link(tgen.gears["r2"])
    switch.add_link(tgen.gears["r3"])


def setup_module(mod):
    "Sets up the pytest environment"
    tgen = Topogen(build_topo, mod.__name__)
    tgen.start_topology()

    router_list = tgen.routers()
    for rname, router in router_list.items():
        router.load_config(
            TopoRouter.RD_ZEBRA, os.path.join(CWD, "{}/zebra.conf".format(rname))
        )
        router.load_config(
            TopoRouter.RD_OSPF, os.path.join(CWD, "{}/ospfd.conf".format(rname))
        )

        # The multicast packet delivery is somewhat controlled by
        # the rp_filter. Setting it to '0' allows the kernel to pass
        # up the mcast packet not destined for the local routers
        # network.
        topotest.sysctl_assure(tgen.net["r1"], "net.ipv4.conf.r1-eth1.rp_filter", 0)
        topotest.sysctl_assure(tgen.net["r1"], "net.ipv4.conf.all.rp_filter", 0)
        topotest.sysctl_assure(tgen.net["r2"], "net.ipv4.conf.r2-eth1.rp_filter", 0)
        topotest.sysctl_assure(tgen.net["r2"], "net.ipv4.conf.all.rp_filter", 0)
        topotest.sysctl_assure(tgen.net["r3"], "net.ipv4.conf.r3-eth1.rp_filter", 0)
        topotest.sysctl_assure(tgen.net["r3"], "net.ipv4.conf.all.rp_filter", 0)

    # Initialize all routers.
    tgen.start_router()
    # tgen.mininet_cli()


def teardown_module():
    "Teardown the pytest environment"
    tgen = get_topogen()
    tgen.stop_topology()


def test_ospf_convergence():
    "Test OSPF daemon convergence and that we have received the ospf routes"
    tgen = get_topogen()
    if tgen.routers_have_failure():
        pytest.skip("skipped because of router(s) failure")

    for router, rnode in tgen.routers().items():
        logger.info('Waiting for router "%s" convergence', router)

        json_file = "{}/{}/ospf-route.json".format(CWD, router)
        expected = json.loads(open(json_file).read())

        test_func = partial(
            topotest.router_json_cmp, rnode, "show ip ospf route json", expected
        )
        _, result = topotest.run_and_expect(test_func, None, count=160, wait=0.5)
        assertmsg = '"{}" JSON output mismatches'.format(router)
        assert result is None, assertmsg
    # tgen.mininet_cli()


def test_ospf_kernel_route():
    "Test OSPF kernel route installation and we have the onlink success"
    tgen = get_topogen()
    if tgen.routers_have_failure():
        pytest.skip("skipped because of router(s) failure")

    rlist = tgen.routers().values()
    for router in rlist:
        logger.info('Checking OSPF IPv4 kernel routes in "%s"', router.name)

        json_file = "{}/{}/v4_route.json".format(CWD, router.name)
        expected = json.loads(open(json_file).read())

        test_func = partial(
            topotest.router_json_cmp, router, "show ip route json", expected
        )
        _, result = topotest.run_and_expect(test_func, None, count=10, wait=0.5)
        assertmsg = '"{}" JSON output mistmatches'.format(router)
        assert result is None, assertmsg
    # tgen.mininet_cli()


def test_memory_leak():
    "Run the memory leak test and report results."
    tgen = get_topogen()
    if not tgen.is_memleak_enabled():
        pytest.skip("Memory leak test/report is disabled")

    tgen.report_memory_leaks()


if __name__ == "__main__":
    args = ["-s"] + sys.argv[1:]
    sys.exit(pytest.main(args))



#!/usr/bin/env python
# SPDX-License-Identifier: ISC

#
# Part of NetDEF Topology Tests
#
# Copyright (c) 2018, LabN Consulting, L.L.C.
# Authored by Lou Berger <lberger@labn.net>
#

import os
import sys
import json
import functools
import pytest

CWD = os.path.dirname(os.path.realpath(__file__))
sys.path.append(os.path.join(CWD, "../"))

# pylint: disable=C0413
# Import topogen and topotest helpers
from lib import topotest
from lib.topogen import Topogen, TopoRouter, get_topogen
from lib.topolog import logger
from lib.common_config import required_linux_kernel_version
from lib.checkping import check_ping

pytestmark = [pytest.mark.bgpd]


def build_topo(tgen):
    tgen.add_router("r1")
    tgen.add_router("r2")
    tgen.add_router("ce1")
    tgen.add_router("ce2")
    tgen.add_router("ce3")
    tgen.add_router("ce4")
    tgen.add_router("ce5")
    tgen.add_router("ce6")

    tgen.add_link(tgen.gears["r1"], tgen.gears["r2"], "eth0", "eth0")
    tgen.add_link(tgen.gears["ce1"], tgen.gears["r1"], "eth0", "eth1")
    tgen.add_link(tgen.gears["ce2"], tgen.gears["r2"], "eth0", "eth1")
    tgen.add_link(tgen.gears["ce3"], tgen.gears["r1"], "eth0", "eth2")
    tgen.add_link(tgen.gears["ce4"], tgen.gears["r2"], "eth0", "eth2")
    tgen.add_link(tgen.gears["ce5"], tgen.gears["r1"], "eth0", "eth3")
    tgen.add_link(tgen.gears["ce6"], tgen.gears["r2"], "eth0", "eth3")


def setup_module(mod):
    result = required_linux_kernel_version("5.15")
    if result is not True:
        pytest.skip("Kernel requirements are not met")

    tgen = Topogen(build_topo, mod.__name__)
    tgen.start_topology()
    for rname, router in tgen.routers().items():
        if os.path.exists("{}/{}/setup.sh".format(CWD, rname)):
            router.run("/bin/bash {}/{}/setup.sh".format(CWD, rname))
        router.load_config(
            TopoRouter.RD_ZEBRA, os.path.join(CWD, "{}/zebra.conf".format(rname))
        )
        router.load_config(
            TopoRouter.RD_BGP, os.path.join(CWD, "{}/bgpd.conf".format(rname))
        )

    tgen.gears["r1"].run("sysctl net.vrf.strict_mode=1")
    tgen.gears["r1"].run("ip link add vrf10 type vrf table 10")
    tgen.gears["r1"].run("ip link set vrf10 up")
    tgen.gears["r1"].run("ip link add vrf20 type vrf table 20")
    tgen.gears["r1"].run("ip link set vrf20 up")
    tgen.gears["r1"].run("ip link set eth1 master vrf10")
    tgen.gears["r1"].run("ip link set eth2 master vrf10")
    tgen.gears["r1"].run("ip link set eth3 master vrf20")

    tgen.gears["r2"].run("sysctl net.vrf.strict_mode=1")
    tgen.gears["r2"].run("ip link add vrf10 type vrf table 10")
    tgen.gears["r2"].run("ip link set vrf10 up")
    tgen.gears["r2"].run("ip link add vrf20 type vrf table 20")
    tgen.gears["r2"].run("ip link set vrf20 up")
    tgen.gears["r2"].run("ip link set eth1 master vrf10")
    tgen.gears["r2"].run("ip link set eth2 master vrf20")
    tgen.gears["r2"].run("ip link set eth3 master vrf20")
    tgen.start_router()


def teardown_module(mod):
    tgen = get_topogen()
    tgen.stop_topology()


def open_json_file(filename):
    try:
        with open(filename, "r") as f:
            return json.load(f)
    except IOError:
        assert False, "Could not read file {}".format(filename)


def check_rib(name, cmd, expected_file):
    def _check(name, dest_addr, match):
        logger.info("polling")
        tgen = get_topogen()
        router = tgen.gears[name]
        output = json.loads(router.vtysh_cmd(cmd))
        expected = open_json_file("{}/{}".format(CWD, expected_file))
        return topotest.json_cmp(output, expected)

    logger.info('[+] check {} "{}" {}'.format(name, cmd, expected_file))
    tgen = get_topogen()
    func = functools.partial(_check, name, cmd, expected_file)
    _, result = topotest.run_and_expect(func, None, count=10, wait=0.5)
    assert result is None, "Failed"


def test_rib():
    check_rib("r1", "show bgp ipv4 vpn json", "r1/vpnv4_rib.json")
    check_rib("r2", "show bgp ipv4 vpn json", "r2/vpnv4_rib.json")
    check_rib("r1", "show ip route vrf vrf10 json", "r1/vrf10_rib.json")
    check_rib("r1", "show ip route vrf vrf20 json", "r1/vrf20_rib.json")
    check_rib("r2", "show ip route vrf vrf10 json", "r2/vrf10_rib.json")
    check_rib("r2", "show ip route vrf vrf20 json", "r2/vrf20_rib.json")
    check_rib("ce1", "show ip route json", "ce1/ip_rib.json")
    check_rib("ce2", "show ip route json", "ce2/ip_rib.json")
    check_rib("ce3", "show ip route json", "ce3/ip_rib.json")
    check_rib("ce4", "show ip route json", "ce4/ip_rib.json")
    check_rib("ce5", "show ip route json", "ce5/ip_rib.json")
    check_rib("ce6", "show ip route json", "ce6/ip_rib.json")


def test_ping():
    check_ping("ce1", "192.168.2.2", True, 10, 0.5)
    check_ping("ce1", "192.168.3.2", True, 10, 0.5)
    check_ping("ce1", "192.168.4.2", False, 10, 0.5)
    check_ping("ce1", "192.168.5.2", False, 10, 0.5)
    check_ping("ce1", "192.168.6.2", False, 10, 0.5)
    check_ping("ce4", "192.168.1.2", False, 10, 0.5)
    check_ping("ce4", "192.168.2.2", False, 10, 0.5)
    check_ping("ce4", "192.168.3.2", False, 10, 0.5)
    check_ping("ce4", "192.168.5.2", True, 10, 0.5)
    check_ping("ce4", "192.168.6.2", True, 10, 0.5)


if __name__ == "__main__":
    args = ["-s"] + sys.argv[1:]
    sys.exit(pytest.main(args))



#!/usr/bin/env python

#
# test_bgp_asdot_regex.py
# Part of Topotests
#
# Copyright 2022 6WIND S.A.
#
# Permission to use, copy, modify, and/or distribute this software
# for any purpose with or without fee is hereby granted, provided
# that the above copyright notice and this permission notice appear
# in all copies.
#
# THE SOFTWARE IS PROVIDED "AS IS" AND NETDEF DISCLAIMS ALL WARRANTIES
# WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
# MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL NETDEF BE LIABLE FOR
# ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY
# DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS,
# WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS
# ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE
# OF THIS SOFTWARE.
#

"""
test_bgp_asdot_regex.py:

Test how regex applies when asnotation to forge bgp config is based on dot or not.
"""

import os
import sys
import json
import pytest
from functools import partial

# add after imports, before defining classes or functions:
pytestmark = [pytest.mark.bgpd]

CWD = os.path.dirname(os.path.realpath(__file__))
sys.path.append(os.path.join(CWD, "../"))

# pylint: disable=C0413
from lib import topotest
from lib.topogen import Topogen, TopoRouter, get_topogen
from lib.topolog import logger


def build_topo(tgen):
    for routern in range(1, 3):
        tgen.add_router("r{}".format(routern))

    switch = tgen.add_switch("s1")
    switch.add_link(tgen.gears["r1"])
    switch.add_link(tgen.gears["r2"])


def setup_module(mod):
    tgen = Topogen(build_topo, mod.__name__)
    tgen.start_topology()

    router_list = tgen.routers()

    for _, (rname, router) in enumerate(router_list.items(), 1):
        router.load_config(
            TopoRouter.RD_ZEBRA, os.path.join(CWD, "{}/zebra.conf".format(rname))
        )
        router.load_config(
            TopoRouter.RD_BGP, os.path.join(CWD, "{}/bgpd.conf".format(rname))
        )

    tgen.start_router()


def teardown_module(mod):
    tgen = get_topogen()
    tgen.stop_topology()


def test_bgp_asdot_regex():
    tgen = get_topogen()

    if tgen.routers_have_failure():
        pytest.skip(tgen.errors)

    router1 = tgen.gears["r1"]
    router2 = tgen.gears["r2"]

    def _bgp_converge(router):
        output = json.loads(router.vtysh_cmd("show ip bgp neighbor 192.168.255.2 json"))
        expected = {
            "192.168.255.2": {
                "bgpState": "Established",
                "addressFamilyInfo": {"ipv4Unicast": {"acceptedPrefixCounter": 1}},
            }
        }
        return topotest.json_cmp(output, expected)

    logger.info("Check if neighbor sessions are up in {}".format(router1.name))
    test_func = partial(_bgp_converge, router1)
    _, result = topotest.run_and_expect(test_func, None, count=15, wait=0.5)
    assert result is None, 'Failed to see BGP convergence in "{}"'.format(router1.name)

    logger.info("BGP neighbor session is up in {}".format(router1.name))

    logger.info("waiting for bgp peers exchanging UPDATES")

    for router in tgen.routers().values():
        ref_file = "{}/{}/show_bgp_ipv4.json".format(CWD, router.name)
        expected = json.loads(open(ref_file).read())
        test_func = partial(
            topotest.router_json_cmp, router, "show bgp ipv4 unicast json", expected
        )
        _, res = topotest.run_and_expect(test_func, None, count=40, wait=2.5)
        assertmsg = "{}: BGP UPDATE exchange failure".format(router.name)
        assert res is None, assertmsg


if __name__ == "__main__":
    args = ["-s"] + sys.argv[1:]
    sys.exit(pytest.main(args))



# SPDX-License-Identifier: ISC
#
# Copyright (c) 2019 by VMware, Inc. ("VMware")
# Used Copyright (c) 2018 by Network Device Education Foundation, Inc.
# ("NetDEF") in this file.
#

import sys
import argparse
from scapy.all import Raw, sendp
import binascii


def send_packet(packet, iface, interval, count):
    """
    Read BSR packet in Raw format and send it to specified interface

    Parameter:
    ---------
    * `packet` : BSR packet in raw format
    * `interface` : Interface from which packet would be send
    * `interval` : Interval between the packets
    * `count` : Number of packets to be sent
    """

    data = binascii.a2b_hex(packet)
    p = Raw(load=data)
    p.show()
    sendp(p, inter=int(interval), iface=iface, count=int(count))


if __name__ == "__main__":
    parser = argparse.ArgumentParser(description="Send BSR Raw packet")
    parser.add_argument("packet", help="Packet in raw format")
    parser.add_argument("iface", help="Packet send to this ineterface")
    parser.add_argument("--interval", help="Interval between packets", default=0)
    parser.add_argument(
        "--count", help="Number of times packet is sent repetitively", default=0
    )
    args = parser.parse_args()

    if not args.packet or not args.iface:
        sys.exit(1)

    send_packet(args.packet, args.iface, args.interval, args.count)



#!/usr/bin/python
# SPDX-License-Identifier: ISC

#
# Copyright (c) 2021 by VMware, Inc. ("VMware")
# Used Copyright (c) 2018 by Network Device Education Foundation, Inc.
# ("NetDEF") in this file.
#


"""OSPF Basic Functionality Automation."""
import os
import sys
import time
import pytest

# Save the Current Working Directory to find configuration files.
CWD = os.path.dirname(os.path.realpath(__file__))
sys.path.append(os.path.join(CWD, "../"))

# pylint: disable=C0413
# Import topogen and topotest helpers
from lib.topogen import Topogen, get_topogen

# Import topoJson from lib, to create topology and initial configuration
from lib.common_config import (
    start_topology,
    write_test_header,
    write_test_footer,
    reset_config_on_routers,
    step,
    scapy_send_raw_packet,
)

from lib.topolog import logger
from lib.topojson import build_config_from_json

from lib.ospf import (
    verify_ospf_neighbor,
    verify_ospf_gr_helper,
    create_router_ospf,
)

pytestmark = [pytest.mark.ospfd]

# Global variables
topo = None
Iters = 5
sw_name = None
intf = None
intf1 = None
pkt = None

"""
Topology:

      Please view in a fixed-width font such as Courier.
      Topo : Broadcast Networks
      DUT - HR      RR
        +---+       +---+          +---+           +---+
        |R0 +       +R1 +          +R2 +           +R3 |
        +-+-+       +-+-+          +-+-+           +-+-+
          |           |              |               |
          |           |              |               |
        --+-----------+--------------+---------------+-----
                         Ethernet Segment

Testcases:

TC1.    Verify by default helper support is disabled for FRR ospf
TC2.    OSPF GR on Broadcast : Verify DUT enters Helper mode when neighbor
        sends grace lsa, helps RR to restart gracefully (RR = DR)
TC3.    OSPF GR on Broadcast : Verify DUT enters Helper mode when neighbor
        sends grace lsa, helps RR to restart gracefully (RR = BDR)
TC4.    OSPF GR on Broadcast : Verify DUT enters Helper mode when neighbor
        sends grace lsa, helps RR to restart gracefully (RR = DRother)
TC5.    OSPF GR on P2P : Verify DUT enters Helper mode when neighbor sends
        grace lsa, helps RR to restart gracefully.
TC6.    Verify all the show commands newly introducted as part of ospf
        helper support - Json Key verification wrt to show commands.
TC7.    Verify helper when grace lsa is received with different configured
        value in process level (higher, lower, grace lsa timer above 1800)
TC8.    Verify helper functionality when dut is helping RR and new grace lsa
        is received from RR.
"""


def setup_module(mod):
    """
    Sets up the pytest environment

    * `mod`: module name
    """
    global topo, intf, intf1, sw_name, pkt
    testsuite_run_time = time.asctime(time.localtime(time.time()))
    logger.info("Testsuite start time: {}".format(testsuite_run_time))
    logger.info("=" * 40)

    logger.info("Running setup_module to create topology")

    # This function initiates the topology build with Topogen...
    json_file = "{}/ospf_gr_helper.json".format(CWD)
    tgen = Topogen(json_file, mod.__name__)
    global topo
    topo = tgen.json_topo
    # ... and here it calls Mininet initialization functions.

    # Starting topology, create tmp files which are loaded to routers
    #  to start daemons and then start routers
    start_topology(tgen)

    # Creating configuration from JSON
    build_config_from_json(tgen, topo)

    # Don't run this test if we have any failure.
    if tgen.routers_have_failure():
        pytest.skip(tgen.errors)

    ospf_covergence = verify_ospf_neighbor(tgen, topo, lan=True)
    assert ospf_covergence is True, "setup_module :Failed \n Error:  {}".format(
        ospf_covergence
    )

    sw_name = "s1"
    intf = topo["routers"]["r0"]["links"][sw_name]["interface"]
    intf1 = topo["routers"]["r1"]["links"][sw_name]["interface"]
    pkt = topo["routers"]["r1"]["opq_lsa_hex"]

    logger.info("Running setup_module() done")


def teardown_module():
    """Teardown the pytest environment"""

    logger.info("Running teardown_module to delete topology")

    tgen = get_topogen()

    try:
        # Stop toplogy and Remove tmp files
        tgen.stop_topology()

    except OSError:
        # OSError exception is raised when mininet tries to stop switch
        # though switch is stopped once but mininet tries to stop same
        # switch again, where it ended up with exception
        pass


def delete_ospf():
    """delete ospf process after each test"""
    tgen = get_topogen()
    step("Delete ospf process")
    for rtr in topo["routers"]:
        ospf_del = {rtr: {"ospf": {"delete": True}}}
        result = create_router_ospf(tgen, topo, ospf_del)
        assert result is True, "Testcase: Failed \n Error: {}".format(result)


# ##################################
# Test cases start here.
# ##################################


def test_ospf_gr_helper_tc7_p1(request):
    """
    Test ospf gr helper
    Verify helper when grace lsa is received with different configured
    value in process level (higher, lower, grace lsa timer above 1800)
    """
    tc_name = request.node.name
    write_test_header(tc_name)
    tgen = get_topogen()

    # Don't run this test if we have any failure.
    if tgen.routers_have_failure():
        pytest.skip(tgen.errors)

    global topo, intf, intf1, pkt

    step("Bring up the base config as per the topology")
    step(
        "Configure DR priority as 99 in RR , DUT dr priority = 98 "
        "& reset ospf process in all the routers"
    )
    step(
        "Enable GR on RR and DUT with grace period on RR = 333"
        "and grace period on DUT = 300"
    )
    reset_config_on_routers(tgen)
    ospf_covergence = verify_ospf_neighbor(tgen, topo, lan=True)
    assert (
        ospf_covergence is True
    ), "OSPF is not after reset config \n Error:  {}".format(ospf_covergence)
    ospf_gr_r0 = {
        "r0": {"ospf": {"graceful-restart": {"helper enable": [], "opaque": True}}}
    }
    result = create_router_ospf(tgen, topo, ospf_gr_r0)
    assert result is True, "Testcase {} : Failed \n Error: {}".format(tc_name, result)

    ospf_gr_r1 = {
        "r1": {"ospf": {"graceful-restart": {"helper enable": [], "opaque": True}}}
    }
    result = create_router_ospf(tgen, topo, ospf_gr_r1)
    assert result is True, "Testcase {} : Failed \n Error: {}".format(tc_name, result)

    input_dict = {"supportedGracePeriod": 1800}
    dut = "r0"
    result = verify_ospf_gr_helper(tgen, topo, dut, input_dict)
    assert result is True, "Testcase {} : Failed \n Error: {}".format(tc_name, result)

    step("Configure grace period = 1801 on RR and restart ospf .")
    grace_period_1801 = "01005e00000570708bd051ef080045c0005cbeb10000015907d111010101e00000050204004801010101000000009714000000000000000000000000000100010209030000000101010180000001c8e9002c000100040000016800020001010000000003000411010101"
    gracelsa_sent = scapy_send_raw_packet(tgen, topo, "r1", intf1, grace_period_1801)

    step("Verify R0 does not enter helper mode.")
    input_dict = {"activeRestarterCnt": 1}
    dut = "r0"
    result = verify_ospf_gr_helper(tgen, topo, dut, input_dict, expected=False)
    assert (
        result is not True
    ), "Testcase {} : Failed. DUT entered helper role   \n Error: {}".format(
        tc_name, result
    )

    delete_ospf()

    write_test_footer(tc_name)


def test_ospf_gr_helper_tc8_p1(request):
    """
    Test ospf gr helper

    Verify helper functionality when dut is helping RR and new grace lsa
    is received from RR.
    """
    tc_name = request.node.name
    write_test_header(tc_name)
    tgen = get_topogen()

    # Don't run this test if we have any failure.
    if tgen.routers_have_failure():
        pytest.skip(tgen.errors)

    global topo, intf, intf1, pkt

    step("Bring up the base config as per the topology")
    step("Enable GR")
    reset_config_on_routers(tgen)
    ospf_covergence = verify_ospf_neighbor(tgen, topo, lan=True)
    assert (
        ospf_covergence is True
    ), "OSPF is not after reset config \n Error:  {}".format(ospf_covergence)
    ospf_gr_r0 = {
        "r0": {"ospf": {"graceful-restart": {"helper enable": [], "opaque": True}}}
    }
    result = create_router_ospf(tgen, topo, ospf_gr_r0)
    assert result is True, "Testcase {} : Failed \n Error: {}".format(tc_name, result)

    ospf_gr_r1 = {
        "r1": {"ospf": {"graceful-restart": {"helper enable": [], "opaque": True}}}
    }
    result = create_router_ospf(tgen, topo, ospf_gr_r1)
    assert result is True, "Testcase {} : Failed \n Error: {}".format(tc_name, result)

    input_dict = {"supportedGracePeriod": 1800}
    dut = "r0"
    result = verify_ospf_gr_helper(tgen, topo, dut, input_dict)
    assert result is True, "Testcase {} : Failed \n Error: {}".format(tc_name, result)

    step("Verify that DUT enters into helper mode.")

    input_dict = {"activeRestarterCnt": 1}
    gracelsa_sent = False
    repeat = 0
    dut = "r0"
    while not gracelsa_sent and repeat < Iters:
        gracelsa_sent = scapy_send_raw_packet(tgen, topo, "r1", intf1, pkt)
        result = verify_ospf_gr_helper(tgen, topo, dut, input_dict)
        if isinstance(result, str):
            repeat += 1
            gracelsa_sent = False

    assert result is True, "Testcase {} : Failed \n Error: {}".format(tc_name, result)

    step("Send the Grace LSA again to DUT when RR is in GR.")
    input_dict = {"activeRestarterCnt": 1}
    gracelsa_sent = False
    repeat = 0
    dut = "r0"
    while not gracelsa_sent and repeat < Iters:
        gracelsa_sent = scapy_send_raw_packet(tgen, topo, "r1", intf1, pkt)
        result = verify_ospf_gr_helper(tgen, topo, dut, input_dict)
        if isinstance(result, str):
            repeat += 1
            gracelsa_sent = False

    assert result is True, "Testcase {} : Failed \n Error: {}".format(tc_name, result)

    delete_ospf()

    write_test_footer(tc_name)


if __name__ == "__main__":
    args = ["-s"] + sys.argv[1:]
    sys.exit(pytest.main(args))



#!/usr/bin/python
# SPDX-License-Identifier: ISC

#
# Copyright (c) 2021 by VMware, Inc. ("VMware")
# Used Copyright (c) 2018 by Network Device Education Foundation, Inc.
# ("NetDEF") in this file.
#


"""OSPF Basic Functionality Automation."""
import os
import sys
import time
import pytest
from time import sleep
import json

pytestmark = pytest.mark.ospf6d

# Save the Current Working Directory to find configuration files.
CWD = os.path.dirname(os.path.realpath(__file__))
sys.path.append(os.path.join(CWD, "../"))
sys.path.append(os.path.join(CWD, "../lib/"))

# pylint: disable=C0413
# Import topogen and topotest helpers
from lib.topogen import Topogen, get_topogen

# Import topoJson from lib, to create topology and initial configuration
from lib.common_config import (
    start_topology,
    write_test_header,
    write_test_footer,
    reset_config_on_routers,
    step,
    shutdown_bringup_interface,
)
from lib.topolog import logger
from lib.topojson import build_config_from_json
from lib.ospf import verify_ospf6_neighbor, config_ospf6_interface
topo = None
# Reading the data from JSON File for topology creation
jsonFile = "{}/ospfv3_authentication.json".format(CWD)
try:
    with open(jsonFile, "r") as topoJson:
        topo = json.load(topoJson)
except IOError:
    assert False, "Could not read file {}".format(jsonFile)
"""
TOPOOLOGY =
      Please view in a fixed-width font such as Courier.
      +---+  A1       +---+
      +R1 +------------+R2 |
      +-+-+-           +--++
        |  --        --  |
        |    -- A0 --    |
      A0|      ----      |
        |      ----      | A2
        |    --    --    |
        |  --        --  |
      +-+-+-            +-+-+
      +R0 +-------------+R3 |
      +---+     A3     +---+

TESTCASES =
1.  OSPFv3 Authentication Trailer - Verify ospfv3 authentication trailer
    using MD5 manual key configuration.
2.  OSPFv3 Authentication Trailer - Verify ospfv3 authentication trailer
    using HMAC-SHA-256 manual key configuration.
3.  OSPFv3 Authentication Trailer - Verify ospfv3 authentication trailer
    using MD5 keychain configuration.
4.  OSPFv3 Authentication Trailer - Verify ospfv3 authentication trailer
    using HMAC-SHA-256 keychain configuration.

 """


def setup_module(mod):
    """
    Sets up the pytest environment
    * `mod`: module name
    """
    testsuite_run_time = time.asctime(time.localtime(time.time()))
    logger.info("Testsuite start time: {}".format(testsuite_run_time))
    logger.info("=" * 40)

    logger.info("Running setup_module to create topology")

    # This function initiates the topology build with Topogen...
    json_file = "{}/ospfv3_single_area.json".format(CWD)
    tgen = Topogen(json_file, mod.__name__)
    global topo
    topo = tgen.json_topo
    # ... and here it calls Mininet initialization functions.

    # Starting topology, create tmp files which are loaded to routers
    #  to start daemons and then start routers
    start_topology(tgen)

    # Creating configuration from JSON
    build_config_from_json(tgen, topo)

    # Don't run this test if we have any failure.
    if tgen.routers_have_failure():
        pytest.skip(tgen.errors)

    ospf6_covergence = verify_ospf6_neighbor(tgen, topo)
    assert ospf6_covergence is True, "setup_module :Failed \n Error:  {}".format(
        ospf6_covergence
    )

    logger.info("Running setup_module() done")


def teardown_module():
    """
    Teardown the pytest environment.
    * `mod`: module name
    """

    logger.info("Running teardown_module to delete topology")

    tgen = get_topogen()

    # Stop toplogy and Remove tmp files
    tgen.stop_topology()

    logger.info(
        "Testsuite end time: {}".format(time.asctime(time.localtime(time.time())))
    )
    logger.info("=" * 40)


# ##################################
# Test cases start here.
# ##################################


def test_ospf6_auth_trailer_tc1_md5(request):
    """
    OSPFv3 Authentication Trailer - Verify ospfv3 authentication trailer
    using MD5 manual key configuration.

    """
    tc_name = request.node.name
    write_test_header(tc_name)
    tgen = get_topogen()
    global topo
    step("Bring up the base config.")
    reset_config_on_routers(tgen)
    step(
        "Configure ospf6 between R1 and R2, enable ospf6 auth on R1 interface "
        "connected to R2 with auth trailer"
    )

    r1_ospf6_auth = {
        "r1": {
            "links": {
                "r2": {
                    "ospf6": {
                        "hash-algo": "md5",
                        "key": "ospf6",
                        "key-id": "10",
                    }
                }
            }
        }
    }
    result = config_ospf6_interface(tgen, topo, r1_ospf6_auth)
    assert result is True, "Testcase {} :Failed \n Error: {}".format(tc_name, result)

    step("Verify that the neighbor is not FULL between R1 and R2.")
    # wait for dead time expiry.
    sleep(6)
    dut = "r1"
    ospf6_covergence = verify_ospf6_neighbor(
        tgen, topo, dut=dut, expected=False, retry_timeout=3
    )
    assert ospf6_covergence is not True, "Testcase {} :Failed \n Error:  {}".format(
        tc_name, ospf6_covergence
    )

    step(
        "Configure ospf6 between R1 and R2, enable ospf6 on R2 interface "
        "connected to R1 with auth trailer"
    )

    r2_ospf6_auth = {
        "r2": {
            "links": {
                "r1": {
                    "ospf6": {
                        "hash-algo": "md5",
                        "key": "ospf6",
                        "key-id": "10",
                    }
                }
            }
        }
    }
    result = config_ospf6_interface(tgen, topo, r2_ospf6_auth)
    assert result is True, "Testcase {} :Failed \n Error: {}".format(tc_name, result)

    step(
        "Verify that the neighbor is FULL between R1 and R2  "
        "using show ipv6 ospf6 neighbor cmd."
    )

    dut = "r2"
    ospf6_covergence = verify_ospf6_neighbor(tgen, topo, dut=dut)
    assert ospf6_covergence is True, "Testcase {} :Failed \n Error:  {}".format(
        tc_name, ospf6_covergence
    )

    step("Disable authentication on R2 ")

    r2_ospf6_auth = {
        "r2": {
            "links": {
                "r1": {
                    "ospf6": {
                        "hash-algo": "md5",
                        "key": "ospf6",
                        "key-id": "10",
                        "del_action": True,
                    }
                }
            }
        }
    }
    result = config_ospf6_interface(tgen, topo, r2_ospf6_auth)
    assert result is True, "Testcase {} :Failed \n Error: {}".format(tc_name, result)

    step("Verify on R1 ,nbr is deleted for R2 after dead interval expiry")
    #  wait till the dead timer expiry
    sleep(6)
    dut = "r2"
    ospf6_covergence = verify_ospf6_neighbor(
        tgen, topo, dut=dut, expected=False, retry_timeout=5
    )
    assert ospf6_covergence is not True, "Testcase {} :Failed \n Error:  {}".format(
        tc_name, ospf6_covergence
    )

    step("Again On R2 enable ospf6 on interface with  message-digest auth")
    r2_ospf6_auth = {
        "r2": {
            "links": {
                "r1": {
                    "ospf6": {
                        "hash-algo": "md5",
                        "key": "ospf6",
                        "key-id": "10",
                    }
                }
            }
        }
    }
    result = config_ospf6_interface(tgen, topo, r2_ospf6_auth)
    assert result is True, "Testcase {} :Failed \n Error: {}".format(tc_name, result)

    step(
        "Verify that the neighbor is FULL between R1 and R2 using"
        " show ip ospf6 neighbor cmd."
    )

    dut = "r2"
    ospf6_covergence = verify_ospf6_neighbor(tgen, topo, dut=dut)
    assert ospf6_covergence is True, "Testcase {} :Failed \n Error:  {}".format(
        tc_name, ospf6_covergence
    )

    step("Shut no shut interface on R1")
    dut = "r1"
    intf = topo["routers"]["r1"]["links"]["r2"]["interface"]
    shutdown_bringup_interface(tgen, dut, intf, False)

    dut = "r2"
    step(
        "Verify that the neighbor is not FULL between R1 and R2 using "
        "show ip ospf6 neighbor cmd."
    )
    ospf6_covergence = verify_ospf6_neighbor(tgen, topo, dut=dut, expected=False)
    assert ospf6_covergence is not True, "Testcase {} :Failed \n Error:  {}".format(
        tc_name, ospf6_covergence
    )

    dut = "r1"
    shutdown_bringup_interface(tgen, dut, intf, True)

    step(
        "Verify that the neighbor is FULL between R1 and R2 using "
        "show ip ospf6 neighbor cmd."
    )

    dut = "r2"
    ospf6_covergence = verify_ospf6_neighbor(tgen, topo, dut=dut)
    assert ospf6_covergence is True, "Testcase {} :Failed \n Error:  {}".format(
        tc_name, ospf6_covergence
    )

    write_test_footer(tc_name)


def test_ospf6_auth_trailer_tc2_sha256(request):
    """
    OSPFv3 Authentication Trailer - Verify ospfv3 authentication trailer
    using HMAC-SHA-256 manual key configuration.

    """
    tc_name = request.node.name
    write_test_header(tc_name)
    tgen = get_topogen()
    global topo
    step("Bring up the base config.")
    reset_config_on_routers(tgen)
    step(
        "Configure ospf6 between R1 and R2, enable ospf6 auth on R1 interface "
        "connected to R2 with auth trailer"
    )

    r1_ospf6_auth = {
        "r1": {
            "links": {
                "r2": {
                    "ospf6": {
                        "hash-algo": "hmac-sha-256",
                        "key": "ospf6",
                        "key-id": "10",
                    }
                }
            }
        }
    }
    result = config_ospf6_interface(tgen, topo, r1_ospf6_auth)
    assert result is True, "Testcase {} :Failed \n Error: {}".format(tc_name, result)

    step("Verify that the neighbor is not FULL between R1 and R2.")
    # wait for dead time expiry.
    sleep(6)
    dut = "r1"
    ospf6_covergence = verify_ospf6_neighbor(
        tgen, topo, dut=dut, expected=False, retry_timeout=3
    )
    assert ospf6_covergence is not True, "Testcase {} :Failed \n Error:  {}".format(
        tc_name, ospf6_covergence
    )

    step(
        "Configure ospf6 between R1 and R2, enable ospf6 on R2 interface "
        "connected to R1 with auth trailer"
    )

    r2_ospf6_auth = {
        "r2": {
            "links": {
                "r1": {
                    "ospf6": {
                        "hash-algo": "hmac-sha-256",
                        "key": "ospf6",
                        "key-id": "10",
                    }
                }
            }
        }
    }
    result = config_ospf6_interface(tgen, topo, r2_ospf6_auth)
    assert result is True, "Testcase {} :Failed \n Error: {}".format(tc_name, result)

    step(
        "Verify that the neighbor is FULL between R1 and R2  "
        "using show ipv6 ospf6 neighbor cmd."
    )

    dut = "r2"
    ospf6_covergence = verify_ospf6_neighbor(tgen, topo, dut=dut)
    assert ospf6_covergence is True, "Testcase {} :Failed \n Error:  {}".format(
        tc_name, ospf6_covergence
    )

    step("Disable authentication on R2 ")

    r2_ospf6_auth = {
        "r2": {
            "links": {
                "r1": {
                    "ospf6": {
                        "hash-algo": "hmac-sha-256",
                        "key": "ospf6",
                        "key-id": "10",
                        "del_action": True,
                    }
                }
            }
        }
    }
    result = config_ospf6_interface(tgen, topo, r2_ospf6_auth)
    assert result is True, "Testcase {} :Failed \n Error: {}".format(tc_name, result)

    step("Verify on R1 ,nbr is deleted for R2 after dead interval expiry")
    #  wait till the dead timer expiry
    sleep(6)
    dut = "r2"
    ospf6_covergence = verify_ospf6_neighbor(
        tgen, topo, dut=dut, expected=False, retry_timeout=5
    )
    assert ospf6_covergence is not True, "Testcase {} :Failed \n Error:  {}".format(
        tc_name, ospf6_covergence
    )

    step("Again On R2 enable ospf6 on interface with  message-digest auth")
    r2_ospf6_auth = {
        "r2": {
            "links": {
                "r1": {
                    "ospf6": {
                        "hash-algo": "hmac-sha-256",
                        "key": "ospf6",
                        "key-id": "10",
                    }
                }
            }
        }
    }
    result = config_ospf6_interface(tgen, topo, r2_ospf6_auth)
    assert result is True, "Testcase {} :Failed \n Error: {}".format(tc_name, result)

    step(
        "Verify that the neighbor is FULL between R1 and R2 using"
        " show ip ospf6 neighbor cmd."
    )

    dut = "r2"
    ospf6_covergence = verify_ospf6_neighbor(tgen, topo, dut=dut)
    assert ospf6_covergence is True, "Testcase {} :Failed \n Error:  {}".format(
        tc_name, ospf6_covergence
    )

    step("Shut no shut interface on R1")
    dut = "r1"
    intf = topo["routers"]["r1"]["links"]["r2"]["interface"]
    shutdown_bringup_interface(tgen, dut, intf, False)

    dut = "r2"
    step(
        "Verify that the neighbor is not FULL between R1 and R2 using "
        "show ip ospf6 neighbor cmd."
    )
    ospf6_covergence = verify_ospf6_neighbor(tgen, topo, dut=dut, expected=False)
    assert ospf6_covergence is not True, "Testcase {} :Failed \n Error:  {}".format(
        tc_name, ospf6_covergence
    )

    dut = "r1"
    shutdown_bringup_interface(tgen, dut, intf, True)

    step(
        "Verify that the neighbor is FULL between R1 and R2 using "
        "show ip ospf6 neighbor cmd."
    )

    dut = "r2"
    ospf6_covergence = verify_ospf6_neighbor(tgen, topo, dut=dut)
    assert ospf6_covergence is True, "Testcase {} :Failed \n Error:  {}".format(
        tc_name, ospf6_covergence
    )

    step("Change the key ID on R2 to not match R1")
    r2_ospf6_auth = {
        "r2": {
            "links": {
                "r1": {
                    "ospf6": {
                        "hash-algo": "hmac-sha-256",
                        "key": "ospf6",
                        "key-id": "30",
                    }
                }
            }
        }
    }
    result = config_ospf6_interface(tgen, topo, r2_ospf6_auth)
    assert result is True, "Testcase {} :Failed \n Error: {}".format(tc_name, result)

    step(
        "Verify on R1 that R2 nbr is deleted due to key-id mismatch "
        "after dead interval expiry"
    )
    #  wait till the dead timer expiry
    sleep(6)
    dut = "r2"
    ospf6_covergence = verify_ospf6_neighbor(
        tgen, topo, dut=dut, expected=False, retry_timeout=5
    )
    assert ospf6_covergence is not True, "Testcase {} :Failed \n Error:  {}".format(
        tc_name, ospf6_covergence
    )

    step("Correct the key ID on R2 so that it matches R1")
    r2_ospf6_auth = {
        "r2": {
            "links": {
                "r1": {
                    "ospf6": {
                        "hash-algo": "hmac-sha-256",
                        "key": "ospf6",
                        "key-id": "10",
                    }
                }
            }
        }
    }
    result = config_ospf6_interface(tgen, topo, r2_ospf6_auth)
    assert result is True, "Testcase {} :Failed \n Error: {}".format(tc_name, result)

    step(
        "Verify that the neighbor is FULL between R1 and R2 using "
        "show ip ospf6 neighbor cmd."
    )

    dut = "r2"
    ospf6_covergence = verify_ospf6_neighbor(tgen, topo, dut=dut)
    assert ospf6_covergence is True, "Testcase {} :Failed \n Error:  {}".format(
        tc_name, ospf6_covergence
    )

    write_test_footer(tc_name)


def test_ospf6_auth_trailer_tc3_keychain_md5(request):
    """
    OSPFv3 Authentication Trailer - Verify ospfv3 authentication trailer
    using MD5 keychain configuration.

    """
    tc_name = request.node.name
    write_test_header(tc_name)
    tgen = get_topogen()
    global topo
    step("Bring up the base config.")
    reset_config_on_routers(tgen)
    step(
        "Configure ospf6 between R1 and R2, enable ospf6 auth on R1 interface "
        "connected to R2 with auth trailer"
    )

    router1 = tgen.gears["r1"]
    router2 = tgen.gears["r2"]

    router1.vtysh_cmd(
        """configure terminal
           key chain auth
             key 10
               key-string ospf6
               cryptographic-algorithm md5"""
    )

    router2.vtysh_cmd(
        """configure terminal
           key chain auth
             key 10
               key-string ospf6
               cryptographic-algorithm md5"""
    )

    r1_ospf6_auth = {
        "r1": {
            "links": {
                "r2": {
                    "ospf6": {
                        "keychain": "auth",
                    }
                }
            }
        }
    }
    result = config_ospf6_interface(tgen, topo, r1_ospf6_auth)
    assert result is True, "Testcase {} :Failed \n Error: {}".format(tc_name, result)

    step("Verify that the neighbor is not FULL between R1 and R2.")
    # wait for dead time expiry.
    sleep(6)
    dut = "r1"
    ospf6_covergence = verify_ospf6_neighbor(
        tgen, topo, dut=dut, expected=False, retry_timeout=3
    )
    assert ospf6_covergence is not True, "Testcase {} :Failed \n Error:  {}".format(
        tc_name, ospf6_covergence
    )

    step(
        "Configure ospf6 between R1 and R2, enable ospf6 on R2 interface "
        "connected to R1 with auth trailer"
    )

    r2_ospf6_auth = {
        "r2": {
            "links": {
                "r1": {
                    "ospf6": {
                        "keychain": "auth",
                    }
                }
            }
        }
    }
    result = config_ospf6_interface(tgen, topo, r2_ospf6_auth)
    assert result is True, "Testcase {} :Failed \n Error: {}".format(tc_name, result)

    step(
        "Verify that the neighbor is FULL between R1 and R2  "
        "using show ipv6 ospf6 neighbor cmd."
    )

    dut = "r2"
    ospf6_covergence = verify_ospf6_neighbor(tgen, topo, dut=dut)
    assert ospf6_covergence is True, "Testcase {} :Failed \n Error:  {}".format(
        tc_name, ospf6_covergence
    )

    step("Disable authentication on R2 ")

    r2_ospf6_auth = {
        "r2": {"links": {"r1": {"ospf6": {"keychain": "auth", "del_action": True}}}}
    }
    result = config_ospf6_interface(tgen, topo, r2_ospf6_auth)
    assert result is True, "Testcase {} :Failed \n Error: {}".format(tc_name, result)

    step("Verify on R1 ,nbr is deleted for R2 after dead interval expiry")
    #  wait till the dead timer expiry
    sleep(6)
    dut = "r2"
    ospf6_covergence = verify_ospf6_neighbor(
        tgen, topo, dut=dut, expected=False, retry_timeout=5
    )
    assert ospf6_covergence is not True, "Testcase {} :Failed \n Error:  {}".format(
        tc_name, ospf6_covergence
    )

    step("Again On R2 enable ospf6 on interface with  message-digest auth")
    r2_ospf6_auth = {
        "r2": {
            "links": {
                "r1": {
                    "ospf6": {
                        "keychain": "auth",
                    }
                }
            }
        }
    }
    result = config_ospf6_interface(tgen, topo, r2_ospf6_auth)
    assert result is True, "Testcase {} :Failed \n Error: {}".format(tc_name, result)

    step(
        "Verify that the neighbor is FULL between R1 and R2 using"
        " show ip ospf6 neighbor cmd."
    )

    dut = "r2"
    ospf6_covergence = verify_ospf6_neighbor(tgen, topo, dut=dut)
    assert ospf6_covergence is True, "Testcase {} :Failed \n Error:  {}".format(
        tc_name, ospf6_covergence
    )

    step("Shut no shut interface on R1")
    dut = "r1"
    intf = topo["routers"]["r1"]["links"]["r2"]["interface"]
    shutdown_bringup_interface(tgen, dut, intf, False)

    dut = "r2"
    step(
        "Verify that the neighbor is not FULL between R1 and R2 using "
        "show ip ospf6 neighbor cmd."
    )
    ospf6_covergence = verify_ospf6_neighbor(tgen, topo, dut=dut, expected=False)
    assert ospf6_covergence is not True, "Testcase {} :Failed \n Error:  {}".format(
        tc_name, ospf6_covergence
    )

    dut = "r1"
    shutdown_bringup_interface(tgen, dut, intf, True)

    step(
        "Verify that the neighbor is FULL between R1 and R2 using "
        "show ip ospf6 neighbor cmd."
    )

    dut = "r2"
    ospf6_covergence = verify_ospf6_neighbor(tgen, topo, dut=dut)
    assert ospf6_covergence is True, "Testcase {} :Failed \n Error:  {}".format(
        tc_name, ospf6_covergence
    )

    write_test_footer(tc_name)


def test_ospf6_auth_trailer_tc4_keychain_sha256(request):
    """
    OSPFv3 Authentication Trailer - Verify ospfv3 authentication trailer
    using HMAC-SHA-256 keychain configuration.

    """
    tc_name = request.node.name
    write_test_header(tc_name)
    tgen = get_topogen()
    global topo
    step("Bring up the base config.")
    reset_config_on_routers(tgen)
    step(
        "Configure ospf6 between R1 and R2, enable ospf6 auth on R1 interface "
        "connected to R2 with auth trailer"
    )

    router1 = tgen.gears["r1"]
    router2 = tgen.gears["r2"]

    router1.vtysh_cmd(
        """configure terminal
           key chain auth
             key 10
               key-string ospf6
               cryptographic-algorithm hmac-sha-256"""
    )

    router2.vtysh_cmd(
        """configure terminal
           key chain auth
             key 10
               key-string ospf6
               cryptographic-algorithm hmac-sha-256"""
    )

    r1_ospf6_auth = {
        "r1": {
            "links": {
                "r2": {
                    "ospf6": {
                        "keychain": "auth",
                    }
                }
            }
        }
    }
    result = config_ospf6_interface(tgen, topo, r1_ospf6_auth)
    assert result is True, "Testcase {} :Failed \n Error: {}".format(tc_name, result)

    step("Verify that the neighbor is not FULL between R1 and R2.")
    # wait for dead time expiry.
    sleep(6)
    dut = "r1"
    ospf6_covergence = verify_ospf6_neighbor(
        tgen, topo, dut=dut, expected=False, retry_timeout=3
    )
    assert ospf6_covergence is not True, "Testcase {} :Failed \n Error:  {}".format(
        tc_name, ospf6_covergence
    )

    step(
        "Configure ospf6 between R1 and R2, enable ospf6 on R2 interface "
        "connected to R1 with auth trailer"
    )

    r2_ospf6_auth = {
        "r2": {
            "links": {
                "r1": {
                    "ospf6": {
                        "keychain": "auth",
                    }
                }
            }
        }
    }
    result = config_ospf6_interface(tgen, topo, r2_ospf6_auth)
    assert result is True, "Testcase {} :Failed \n Error: {}".format(tc_name, result)

    step(
        "Verify that the neighbor is FULL between R1 and R2  "
        "using show ipv6 ospf6 neighbor cmd."
    )

    dut = "r2"
    ospf6_covergence = verify_ospf6_neighbor(tgen, topo, dut=dut)
    assert ospf6_covergence is True, "Testcase {} :Failed \n Error:  {}".format(
        tc_name, ospf6_covergence
    )

    step("Disable authentication on R2 ")

    r2_ospf6_auth = {
        "r2": {"links": {"r1": {"ospf6": {"keychain": "auth", "del_action": True}}}}
    }
    result = config_ospf6_interface(tgen, topo, r2_ospf6_auth)
    assert result is True, "Testcase {} :Failed \n Error: {}".format(tc_name, result)

    step("Verify on R1 ,nbr is deleted for R2 after dead interval expiry")
    #  wait till the dead timer expiry
    sleep(6)
    dut = "r2"
    ospf6_covergence = verify_ospf6_neighbor(
        tgen, topo, dut=dut, expected=False, retry_timeout=5
    )
    assert ospf6_covergence is not True, "Testcase {} :Failed \n Error:  {}".format(
        tc_name, ospf6_covergence
    )

    step("Again On R2 enable ospf6 on interface with  message-digest auth")
    r2_ospf6_auth = {
        "r2": {
            "links": {
                "r1": {
                    "ospf6": {
                        "keychain": "auth",
                    }
                }
            }
        }
    }
    result = config_ospf6_interface(tgen, topo, r2_ospf6_auth)
    assert result is True, "Testcase {} :Failed \n Error: {}".format(tc_name, result)

    step(
        "Verify that the neighbor is FULL between R1 and R2 using"
        " show ip ospf6 neighbor cmd."
    )

    dut = "r2"
    ospf6_covergence = verify_ospf6_neighbor(tgen, topo, dut=dut)
    assert ospf6_covergence is True, "Testcase {} :Failed \n Error:  {}".format(
        tc_name, ospf6_covergence
    )

    step("Shut no shut interface on R1")
    dut = "r1"
    intf = topo["routers"]["r1"]["links"]["r2"]["interface"]
    shutdown_bringup_interface(tgen, dut, intf, False)

    dut = "r2"
    step(
        "Verify that the neighbor is not FULL between R1 and R2 using "
        "show ip ospf6 neighbor cmd."
    )
    ospf6_covergence = verify_ospf6_neighbor(tgen, topo, dut=dut, expected=False)
    assert ospf6_covergence is not True, "Testcase {} :Failed \n Error:  {}".format(
        tc_name, ospf6_covergence
    )

    dut = "r1"
    shutdown_bringup_interface(tgen, dut, intf, True)

    step(
        "Verify that the neighbor is FULL between R1 and R2 using "
        "show ip ospf6 neighbor cmd."
    )

    dut = "r2"
    ospf6_covergence = verify_ospf6_neighbor(tgen, topo, dut=dut)
    assert ospf6_covergence is True, "Testcase {} :Failed \n Error:  {}".format(
        tc_name, ospf6_covergence
    )

    write_test_footer(tc_name)


def test_ospf6_auth_trailer_tc5_md5_keymissmatch(request):
    """
    OSPFv3 Authentication Trailer - Verify ospfv3 authentication trailer
    using MD5 manual key configuration.

    """
    tc_name = request.node.name
    write_test_header(tc_name)
    tgen = get_topogen()
    global topo
    step("Bring up the base config.")
    reset_config_on_routers(tgen)
    step(
        "Configure ospf6 between R1 and R2, enable ospf6 auth on R1 interface "
        "connected to R2 with auth trailer"
    )

    r1_ospf6_auth = {
        "r1": {
            "links": {
                "r2": {
                    "ospf6": {
                        "hash-algo": "md5",
                        "key": "ospf6",
                        "key-id": "10",
                    }
                }
            }
        }
    }
    result = config_ospf6_interface(tgen, topo, r1_ospf6_auth)
    assert result is True, "Testcase {} :Failed \n Error: {}".format(tc_name, result)

    step("Verify that the neighbor is not FULL between R1 and R2.")
    # wait for dead time expiry.
    sleep(6)
    dut = "r1"
    ospf6_covergence = verify_ospf6_neighbor(
        tgen, topo, dut=dut, expected=False, retry_timeout=3
    )
    assert ospf6_covergence is not True, "Testcase {} :Failed \n Error:  {}".format(
        tc_name, ospf6_covergence
    )

    step(
        "Configure ospf6 between R1 and R2, enable ospf6 on R2 interface "
        "connected to R1 with auth trailer wrong key"
    )

    r2_ospf6_auth = {
        "r2": {
            "links": {
                "r1": {
                    "ospf6": {
                        "hash-algo": "md5",
                        "key": "ospf6-missmatch",
                        "key-id": "10",
                    }
                }
            }
        }
    }
    result = config_ospf6_interface(tgen, topo, r2_ospf6_auth)
    assert result is True, "Testcase {} :Failed \n Error: {}".format(tc_name, result)

    step(
        "Verify that the neighbor is not FULL between R1 and R2  "
        "using show ipv6 ospf6 neighbor cmd."
    )

    step("Verify that the neighbor is FULL between R1 and R2.")
    # wait for dead time expiry.
    sleep(6)
    dut = "r2"
    ospf6_covergence = verify_ospf6_neighbor(
        tgen, topo, dut=dut, expected=False, retry_timeout=3
    )
    assert ospf6_covergence is not True, "Testcase {} :Failed \n Error:  {}".format(
        tc_name, ospf6_covergence
    )

    step(
        "Configure ospf6 between R1 and R2, enable ospf6 on R2 interface "
        "connected to R1 with auth trailer correct key"
    )

    r2_ospf6_auth = {
        "r2": {
            "links": {
                "r1": {
                    "ospf6": {
                        "hash-algo": "md5",
                        "key": "ospf6",
                        "key-id": "10",
                    }
                }
            }
        }
    }
    result = config_ospf6_interface(tgen, topo, r2_ospf6_auth)
    assert result is True, "Testcase {} :Failed \n Error: {}".format(tc_name, result)

    step(
        "Verify that the neighbor is FULL between R1 and R2  "
        "using show ipv6 ospf6 neighbor cmd."
    )

    dut = "r2"
    ospf6_covergence = verify_ospf6_neighbor(tgen, topo, dut=dut)
    assert ospf6_covergence is True, "Testcase {} :Failed \n Error:  {}".format(
        tc_name, ospf6_covergence
    )

    write_test_footer(tc_name)


def test_ospf6_auth_trailer_tc6_sha256_mismatch(request):
    """
    OSPFv3 Authentication Trailer - Verify ospfv3 authentication trailer
    using HMAC-SHA-256 manual key configuration.

    """
    tc_name = request.node.name
    write_test_header(tc_name)
    tgen = get_topogen()
    global topo
    step("Bring up the base config.")
    reset_config_on_routers(tgen)
    step(
        "Configure ospf6 between R1 and R2, enable ospf6 auth on R1 interface "
        "connected to R2 with auth trailer"
    )

    r1_ospf6_auth = {
        "r1": {
            "links": {
                "r2": {
                    "ospf6": {
                        "hash-algo": "hmac-sha-256",
                        "key": "ospf6",
                        "key-id": "10",
                    }
                }
            }
        }
    }
    result = config_ospf6_interface(tgen, topo, r1_ospf6_auth)
    assert result is True, "Testcase {} :Failed \n Error: {}".format(tc_name, result)

    step("Verify that the neighbor is not FULL between R1 and R2.")
    # wait for dead time expiry.
    sleep(6)
    dut = "r1"
    ospf6_covergence = verify_ospf6_neighbor(
        tgen, topo, dut=dut, expected=False, retry_timeout=3
    )
    assert ospf6_covergence is not True, "Testcase {} :Failed \n Error:  {}".format(
        tc_name, ospf6_covergence
    )

    step(
        "Configure ospf6 with on R1 and R2, enable ospf6 on R2 interface "
        "connected to R1 with auth trailer wrong key"
    )

    r2_ospf6_auth = {
        "r2": {
            "links": {
                "r1": {
                    "ospf6": {
                        "hash-algo": "hmac-sha-256",
                        "key": "ospf6-missmatch",
                        "key-id": "10",
                    }
                }
            }
        }
    }
    result = config_ospf6_interface(tgen, topo, r2_ospf6_auth)
    assert result is True, "Testcase {} :Failed \n Error: {}".format(tc_name, result)

    step("Verify that the neighbor is not FULL between R1 and R2.")
    # wait for dead time expiry.
    sleep(6)
    dut = "r2"
    ospf6_covergence = verify_ospf6_neighbor(
        tgen, topo, dut=dut, expected=False, retry_timeout=3
    )
    assert ospf6_covergence is not True, "Testcase {} :Failed \n Error:  {}".format(
        tc_name, ospf6_covergence
    )

    step(
        "Configure ospf6 with on R1 and R2, enable ospf6 on R2 interface "
        "connected to R1 with auth trailer wrong key"
    )

    r2_ospf6_auth = {
        "r2": {
            "links": {
                "r1": {
                    "ospf6": {
                        "hash-algo": "hmac-sha-256",
                        "key": "ospf6",
                        "key-id": "10",
                    }
                }
            }
        }
    }
    result = config_ospf6_interface(tgen, topo, r2_ospf6_auth)
    assert result is True, "Testcase {} :Failed \n Error: {}".format(tc_name, result)

    step(
        "Verify that the neighbor is FULL between R1 and R2  "
        "using show ipv6 ospf6 neighbor cmd."
    )

    dut = "r2"
    ospf6_covergence = verify_ospf6_neighbor(tgen, topo, dut=dut)
    assert ospf6_covergence is True, "Testcase {} :Failed \n Error:  {}".format(
        tc_name, ospf6_covergence
    )

    write_test_footer(tc_name)


def test_ospf6_auth_trailer_tc7_keychain_md5_missmatch(request):
    """
    OSPFv3 Authentication Trailer - Verify ospfv3 authentication trailer
    using MD5 keychain configuration.

    """
    tc_name = request.node.name
    write_test_header(tc_name)
    tgen = get_topogen()
    global topo
    step("Bring up the base config.")
    reset_config_on_routers(tgen)
    step(
        "Configure ospf6 between R1 and R2, enable ospf6 auth on R1 interface "
        "connected to R2 with auth trailer"
    )

    router1 = tgen.gears["r1"]
    router2 = tgen.gears["r2"]

    router1.vtysh_cmd(
        """configure terminal
           key chain auth
             key 10
               key-string ospf6
               cryptographic-algorithm md5"""
    )

    router2.vtysh_cmd(
        """configure terminal
           key chain auth
             key 10
               key-string ospf6
               cryptographic-algorithm md5"""
    )

    router2.vtysh_cmd(
        """configure terminal
           key chain auth-missmatch
             key 10
               key-string ospf6-missmatch
               cryptographic-algorithm md5"""
    )

    r1_ospf6_auth = {
        "r1": {
            "links": {
                "r2": {
                    "ospf6": {
                        "keychain": "auth",
                    }
                }
            }
        }
    }
    result = config_ospf6_interface(tgen, topo, r1_ospf6_auth)
    assert result is True, "Testcase {} :Failed \n Error: {}".format(tc_name, result)

    step("Verify that the neighbor is not FULL between R1 and R2.")
    # wait for dead time expiry.
    sleep(6)
    dut = "r1"
    ospf6_covergence = verify_ospf6_neighbor(
        tgen, topo, dut=dut, expected=False, retry_timeout=3
    )
    assert ospf6_covergence is not True, "Testcase {} :Failed \n Error:  {}".format(
        tc_name, ospf6_covergence
    )

    step(
        "Configure ospf6 between R1 and R2, enable ospf6 on R2 interface "
        "connected to R1 with auth trailer with wrong keychain"
    )

    r2_ospf6_auth = {
        "r2": {
            "links": {
                "r1": {
                    "ospf6": {
                        "keychain": "auth-missmatch",
                    }
                }
            }
        }
    }
    result = config_ospf6_interface(tgen, topo, r2_ospf6_auth)
    assert result is True, "Testcase {} :Failed \n Error: {}".format(tc_name, result)

    step("Verify that the neighbor is not FULL between R1 and R2.")
    # wait for dead time expiry.
    sleep(6)
    dut = "r2"
    ospf6_covergence = verify_ospf6_neighbor(
        tgen, topo, dut=dut, expected=False, retry_timeout=3
    )
    assert ospf6_covergence is not True, "Testcase {} :Failed \n Error:  {}".format(
        tc_name, ospf6_covergence
    )

    step(
        "Configure ospf6 between R1 and R2, enable ospf6 on R2 interface "
        "connected to R1 with auth trailer with correct keychain"
    )

    r2_ospf6_auth = {
        "r2": {
            "links": {
                "r1": {
                    "ospf6": {
                        "keychain": "auth",
                    }
                }
            }
        }
    }
    result = config_ospf6_interface(tgen, topo, r2_ospf6_auth)
    assert result is True, "Testcase {} :Failed \n Error: {}".format(tc_name, result)

    step(
        "Verify that the neighbor is FULL between R1 and R2  "
        "using show ipv6 ospf6 neighbor cmd."
    )

    dut = "r2"
    ospf6_covergence = verify_ospf6_neighbor(tgen, topo, dut=dut)
    assert ospf6_covergence is True, "Testcase {} :Failed \n Error:  {}".format(
        tc_name, ospf6_covergence
    )

    write_test_footer(tc_name)


def test_ospf6_auth_trailer_tc8_keychain_sha256_missmatch(request):
    """
    OSPFv3 Authentication Trailer - Verify ospfv3 authentication trailer
    using HMAC-SHA-256 keychain configuration.

    """
    tc_name = request.node.name
    write_test_header(tc_name)
    tgen = get_topogen()
    global topo
    step("Bring up the base config.")
    reset_config_on_routers(tgen)
    step(
        "Configure ospf6 between R1 and R2, enable ospf6 auth on R1 interface "
        "connected to R2 with auth trailer"
    )

    router1 = tgen.gears["r1"]
    router2 = tgen.gears["r2"]

    router1.vtysh_cmd(
        """configure terminal
           key chain auth
             key 10
               key-string ospf6
               cryptographic-algorithm hmac-sha-256"""
    )

    router2.vtysh_cmd(
        """configure terminal
           key chain auth
             key 10
               key-string ospf6
               cryptographic-algorithm hmac-sha-256"""
    )

    router2.vtysh_cmd(
        """configure terminal
           key chain auth-missmatch
             key 10
               key-string ospf6-missmatch
               cryptographic-algorithm hmac-sha-256"""
    )

    r1_ospf6_auth = {
        "r1": {
            "links": {
                "r2": {
                    "ospf6": {
                        "keychain": "auth",
                    }
                }
            }
        }
    }
    result = config_ospf6_interface(tgen, topo, r1_ospf6_auth)
    assert result is True, "Testcase {} :Failed \n Error: {}".format(tc_name, result)

    step("Verify that the neighbor is not FULL between R1 and R2.")
    # wait for dead time expiry.
    sleep(6)
    dut = "r1"
    ospf6_covergence = verify_ospf6_neighbor(
        tgen, topo, dut=dut, expected=False, retry_timeout=3
    )
    assert ospf6_covergence is not True, "Testcase {} :Failed \n Error:  {}".format(
        tc_name, ospf6_covergence
    )

    step(
        "Configure ospf6 between R1 and R2, enable ospf6 on R2 interface "
        "connected to R1 with auth trailer wrong keychain"
    )

    r2_ospf6_auth = {
        "r2": {
            "links": {
                "r1": {
                    "ospf6": {
                        "keychain": "auth-missmatch",
                    }
                }
            }
        }
    }
    result = config_ospf6_interface(tgen, topo, r2_ospf6_auth)
    assert result is True, "Testcase {} :Failed \n Error: {}".format(tc_name, result)

    step("Verify that the neighbor is not FULL between R1 and R2.")
    # wait for dead time expiry.
    sleep(6)
    dut = "r2"
    ospf6_covergence = verify_ospf6_neighbor(
        tgen, topo, dut=dut, expected=False, retry_timeout=3
    )
    assert ospf6_covergence is not True, "Testcase {} :Failed \n Error:  {}".format(
        tc_name, ospf6_covergence
    )

    step(
        "Configure ospf6 between R1 and R2, enable ospf6 on R2 interface "
        "connected to R1 with auth trailer correct keychain"
    )

    r2_ospf6_auth = {
        "r2": {
            "links": {
                "r1": {
                    "ospf6": {
                        "keychain": "auth",
                    }
                }
            }
        }
    }
    result = config_ospf6_interface(tgen, topo, r2_ospf6_auth)
    assert result is True, "Testcase {} :Failed \n Error: {}".format(tc_name, result)

    step(
        "Verify that the neighbor is FULL between R1 and R2  "
        "using show ipv6 ospf6 neighbor cmd."
    )

    dut = "r2"
    ospf6_covergence = verify_ospf6_neighbor(tgen, topo, dut=dut)
    assert ospf6_covergence is True, "Testcase {} :Failed \n Error:  {}".format(
        tc_name, ospf6_covergence
    )

    write_test_footer(tc_name)


def test_ospf6_auth_trailer_tc9_keychain_not_configured(request):
    """
    OSPFv3 Neighborship without Authentication Trailer -
    Verify ospfv3 neighborship when no authentication trailer is configured.

    """
    tc_name = request.node.name
    write_test_header(tc_name)
    tgen = get_topogen()
    global topo
    step("Bring up the base config.")
    reset_config_on_routers(tgen)
    step(
        "Configure ospf6 between R1 and R2, enable ospf6 auth on R1 interface "
        "connected to R2 with auth trailer"
    )

    router1 = tgen.gears["r1"]
    router2 = tgen.gears["r2"]

    r1_ospf6_auth = {
        "r1": {
            "links": {
                "r2": {
                    "ospf6": {
                        "keychain": "auth",
                    }
                }
            }
        }
    }
    result = config_ospf6_interface(tgen, topo, r1_ospf6_auth)
    assert result is True, "Testcase {} :Failed \n Error: {}".format(tc_name, result)

    step("Verify that the neighbor is not FULL between R1 and R2.")
    # wait for dead time expiry.
    sleep(6)
    dut = "r1"
    ospf6_covergence = verify_ospf6_neighbor(
        tgen, topo, dut=dut, expected=False, retry_timeout=3
    )
    assert ospf6_covergence is not True, "Testcase {} :Failed \n Error:  {}".format(
        tc_name, ospf6_covergence
    )

    step(
        "Configure ospf6 between R1 and R2, enable ospf6 on R2 interface "
        "connected to R1 with auth trailer non existing keychain"
    )

    r2_ospf6_auth = {
        "r2": {
            "links": {
                "r1": {
                    "ospf6": {
                        "keychain": "auth",
                    }
                }
            }
        }
    }
    result = config_ospf6_interface(tgen, topo, r2_ospf6_auth)
    assert result is True, "Testcase {} :Failed \n Error: {}".format(tc_name, result)

    step("Verify that the neighbor is not FULL between R1 and R2.")
    # wait for dead time expiry.
    sleep(6)
    dut = "r2"
    ospf6_covergence = verify_ospf6_neighbor(
        tgen, topo, dut=dut, expected=False, retry_timeout=3
    )
    assert ospf6_covergence is not True, "Testcase {} :Failed \n Error:  {}".format(
        tc_name, ospf6_covergence
    )

    write_test_footer(tc_name)


def test_ospf6_auth_trailer_tc10_no_auth_trailer(request):
    """
    OSPFv3 Neighborship without Authentication Trailer -
    Verify ospfv3 neighborship when no authentication trailer is configured.

    """
    tc_name = request.node.name
    write_test_header(tc_name)
    tgen = get_topogen()
    global topo
    step("Bring up the base config.")
    reset_config_on_routers(tgen)

    router1 = tgen.gears["r1"]
    router2 = tgen.gears["r2"]

    step(
        "Verify that the neighbor is FULL between R1 and R2  "
        "using show ipv6 ospf6 neighbor cmd."
    )

    dut = "r2"
    ospf6_covergence = verify_ospf6_neighbor(tgen, topo, dut=dut)
    assert ospf6_covergence is True, "Testcase {} :Failed \n Error:  {}".format(
        tc_name, ospf6_covergence
    )

    write_test_footer(tc_name)


if __name__ == "__main__":
    args = ["-s"] + sys.argv[1:]
    sys.exit(pytest.main(args))



#!/usr/bin/env python
# SPDX-License-Identifier: ISC

#
# test_ospf_multi_instance.py
#
# Copyright (c) 2024 LabN Consulting
# Acee Lindem
#

import os
import sys
from functools import partial
import pytest

# pylint: disable=C0413
# Import topogen and topotest helpers
from lib import topotest
from lib.topogen import Topogen, TopoRouter, get_topogen
from lib.topolog import logger

from lib.common_config import (
    step,
    create_interface_in_kernel,
)


"""
test_ospf_metric_propagation.py: Test OSPF/BGP metric propagation
"""

TOPOLOGY = """

 +---------+             +--------------------+             +---------+
 |   r1    |             |  r2      |   r2    |             |   r3    |
 |         |             | ospf 1   | ospf 2  |             |        |
 | 1.1.1.1 | eth0    eth0| 2.2.2.1  | 2.2.2.2 |eth1     eth0| 3.3.3.1 |
 |         +-------------+          |         +-------------+         |
 |         | 10.1.1.0/24 |          |         | 10.1.2.0/24 |         |
 +---------+             +--------------------+             +---------+


"""

# Save the Current Working Directory to find configuration files.
CWD = os.path.dirname(os.path.realpath(__file__))
sys.path.append(os.path.join(CWD, "../"))

# Required to instantiate the topology builder class.

pytestmark = [pytest.mark.ospfd, pytest.mark.bgpd]


def build_topo(tgen):
    "Build function"

    # Create 3 routers
    tgen.add_router("r1")
    tgen.add_router("r2")
    tgen.add_router("r3")

    # Interconect router 1, 2 (0)
    switch = tgen.add_switch("s1-1-2")
    switch.add_link(tgen.gears["r1"])
    switch.add_link(tgen.gears["r2"])

    # Interconect router 2, 3 (1)
    switch = tgen.add_switch("s2-2-3")
    switch.add_link(tgen.gears["r2"])
    switch.add_link(tgen.gears["r3"])

    # Add more loopbacks to r2
    create_interface_in_kernel(
        tgen, "r2", "lo1", "2.2.2.1", netmask="255.255.255.255", create=True
    )
    create_interface_in_kernel(
        tgen, "r2", "lo2", "2.2.2.2", netmask="255.255.255.255", create=True
    )


def setup_module(mod):
    logger.info("OSPF Multi-Instance:\n {}".format(TOPOLOGY))

    tgen = Topogen(build_topo, mod.__name__)
    tgen.start_topology()

    # Starting Routers
    router_list = tgen.routers()

    for rname, router in router_list.items():
        logger.info("Loading router %s" % rname)
        router.load_frr_config(os.path.join(CWD, "{}/frr.conf".format(rname)))

    # Initialize all routers.
    tgen.start_router()


def teardown_module():
    "Teardown the pytest environment"
    tgen = get_topogen()
    tgen.stop_topology()


def test_multi_instance_default_origination():
    tgen = get_topogen()

    if tgen.routers_have_failure():
        pytest.skip("Skipped because of router(s) failure")

    step("Configure a local default route")
    r1 = tgen.gears["r1"]
    r1.vtysh_cmd("conf t\nip route 0.0.0.0/0 Null0")

    step("Verify the R1 configuration and install of 'ip route 0.0.0.0/0 Null0'")
    prefix_suppression_cfg = (
        tgen.net["r1"]
        .cmd('vtysh -c "show running" | grep "^ip route 0.0.0.0/0 Null0"')
        .rstrip()
    )
    assertmsg = "'ip route 0.0.0.0/0 Null0' applied, but not present in configuration"
    assert prefix_suppression_cfg == "ip route 0.0.0.0/0 Null0", assertmsg

    input_dict = {
        "0.0.0.0/0": [
            {
                "prefix": "0.0.0.0/0",
                "prefixLen": 0,
                "protocol": "static",
                "nexthops": [
                    {
                        "blackhole": True,
                    }
                ],
            }
        ]
    }
    test_func = partial(
        topotest.router_json_cmp, r1, "show ip route 0.0.0.0/0 json", input_dict
    )
    _, result = topotest.run_and_expect(test_func, None, count=60, wait=1)
    assertmsg = "0.0.0.0/0 not installed on router r1"
    assert result is None, assertmsg

    step(
        "Verify the R1 configuration and advertisement of 'default-information originate'"
    )
    r1.vtysh_cmd("conf t\nrouter ospf\n default-information originate")

    input_dict = {
        "asExternalLinkStates": [
            {
                "lsaType": "AS-external-LSA",
                "linkStateId": "0.0.0.0",
                "advertisingRouter": "1.1.1.1",
                "networkMask": 0,
                "metricType": "E2 (Larger than any link state path)",
                "metric": 10,
                "forwardAddress": "0.0.0.0",
                "externalRouteTag": 0,
            }
        ]
    }
    test_func = partial(
        topotest.router_json_cmp, r1, "show ip ospf database json", input_dict
    )

    r2 = tgen.gears["r2"]
    step("Verify the OSPF instance 1 installation of default route on router 2")
    input_dict = {
        "0.0.0.0/0": [
            {
                "prefix": "0.0.0.0/0",
                "prefixLen": 0,
                "protocol": "ospf",
                "instance": 1,
                "nexthops": [
                    {
                        "ip": "10.1.1.1",
                        "interfaceName": "r2-eth0",
                    }
                ],
            }
        ]
    }
    test_func = partial(
        topotest.router_json_cmp, r2, "show ip route 0.0.0.0/0 json", input_dict
    )
    _, result = topotest.run_and_expect(test_func, None, count=60, wait=1)
    assertmsg = "0.0.0.0/0 not installed on router r2"
    assert result is None, assertmsg

    step("Configure OSPF 'default-intformation originate' on router r2 instance 2")
    r2.vtysh_cmd("conf t\nrouter ospf 2\n default-information originate")

    step("Verify r2 instance 2 AS-External default origination")
    input_dict = {
        "ospfInstance": 2,
        "routerId": "2.2.2.2",
        "asExternalLinkStates": [
            {
                "lsaType": "AS-external-LSA",
                "linkStateId": "0.0.0.0",
                "advertisingRouter": "2.2.2.2",
                "networkMask": 0,
                "metricType": "E2 (Larger than any link state path)",
                "tos": 0,
                "metric": 10,
                "forwardAddress": "0.0.0.0",
                "externalRouteTag": 0,
            }
        ],
    }
    test_func = partial(
        topotest.router_json_cmp,
        r2,
        "show ip ospf 2 database external json",
        input_dict,
    )
    _, result = topotest.run_and_expect(test_func, None, count=60, wait=1)
    assertmsg = "AS-External default not originated by router r2 OSPF instance 2"
    assert result is None, assertmsg

    step("Update the OSPF instance 2 distance so it will be preferred over instance 1")
    r2.vtysh_cmd("conf t\nrouter ospf 2\n distance 15")

    step("Generate a default route from OSPF on r3")
    r3 = tgen.gears["r3"]
    r3.vtysh_cmd("conf t\nrouter ospf\n default-information originate")
    r3.vtysh_cmd("conf t\nip route 0.0.0.0/0 Null0")

    step("Verify r3 AS-External default origination on r2")
    input_dict = {
        "ospfInstance": 2,
        "routerId": "2.2.2.2",
        "asExternalLinkStates": [
            {
                "lsaType": "AS-external-LSA",
                "linkStateId": "0.0.0.0",
                "advertisingRouter": "3.3.3.1",
                "length": 36,
                "networkMask": 0,
                "metricType": "E2 (Larger than any link state path)",
                "tos": 0,
                "metric": 10,
                "forwardAddress": "0.0.0.0",
                "externalRouteTag": 0,
            }
        ],
    }
    test_func = partial(
        topotest.router_json_cmp,
        r2,
        "show ip ospf 2 database external json",
        input_dict,
    )
    _, result = topotest.run_and_expect(test_func, None, count=60, wait=1)
    assertmsg = "AS-External default not originated by router r3 OSPF"
    assert result is None, assertmsg

    step("Verify r3's default installed by OSPF instance 2 is preferred on r2")
    input_dict = {
        "0.0.0.0/0": [
            {
                "prefix": "0.0.0.0/0",
                "prefixLen": 0,
                "protocol": "ospf",
                "instance": 2,
                "distance": 15,
                "nexthops": [
                    {
                        "ip": "10.1.2.3",
                        "interfaceName": "r2-eth1",
                    }
                ],
            }
        ]
    }
    test_func = partial(
        topotest.router_json_cmp, r2, "show ip route 0.0.0.0/0 json", input_dict
    )
    _, result = topotest.run_and_expect(test_func, None, count=60, wait=1)
    assertmsg = "0.0.0.0/0 from r3 not installed on router r2"
    assert result is None, assertmsg

    step(
        "Verify that r2's OSPF instance 2 AS-External LSA default is flushed due to default from r3"
    )
    input_dict = {
        "ospfInstance": 2,
        "routerId": "2.2.2.2",
        "asExternalLinkStates": [
            {
                "lsaAge": 3600,
                "lsaType": "AS-external-LSA",
                "linkStateId": "0.0.0.0",
                "advertisingRouter": "2.2.2.2",
                "networkMask": 0,
                "metricType": "E2 (Larger than any link state path)",
                "tos": 0,
                "metric": 10,
                "forwardAddress": "0.0.0.0",
                "externalRouteTag": 0,
            }
        ],
    }
    test_func = partial(
        topotest.router_json_cmp,
        r2,
        "show ip ospf 2 database external json",
        input_dict,
    )
    _, result = topotest.run_and_expect(test_func, None, count=60, wait=1)
    assertmsg = "AS-External default not flushed by router r2 OSPF instance 2"
    assert result is None, assertmsg

    step("Remove r3's default route and verify that its advertisement is flushed")
    r3.vtysh_cmd("conf t\nno ip route 0.0.0.0/0 Null0")
    input_dict = {
        "routerId": "3.3.3.1",
        "asExternalLinkStates": [
            {
                "lsaAge": 3600,
                "lsaType": "AS-external-LSA",
                "linkStateId": "0.0.0.0",
                "advertisingRouter": "3.3.3.1",
                "networkMask": 0,
                "metricType": "E2 (Larger than any link state path)",
                "tos": 0,
                "metric": 10,
                "forwardAddress": "0.0.0.0",
                "externalRouteTag": 0,
            }
        ],
    }
    test_func = partial(
        topotest.router_json_cmp, r3, "show ip ospf database external json", input_dict
    )
    _, result = topotest.run_and_expect(test_func, None, count=60, wait=1)
    assertmsg = "AS-External default not flushed by router r3 OSPF"
    assert result is None, assertmsg

    step(
        "Verify that r2's OSPF instance 2 AS-External default is advertised and installed by r3"
    )
    input_dict = {
        "routerId": "3.3.3.1",
        "asExternalLinkStates": [
            {
                "lsaType": "AS-external-LSA",
                "linkStateId": "0.0.0.0",
                "advertisingRouter": "2.2.2.2",
                "networkMask": 0,
                "metricType": "E2 (Larger than any link state path)",
                "tos": 0,
                "metric": 10,
                "forwardAddress": "0.0.0.0",
                "externalRouteTag": 0,
            }
        ],
    }
    test_func = partial(
        topotest.router_json_cmp, r3, "show ip ospf database external json", input_dict
    )
    assertmsg = "AS-External default not originated by r2 OSPF instance 2"
    assert result is None, assertmsg

    step("Verify r2's OSPF instance 2 is AS-External default is installed on r3")
    input_dict = {
        "0.0.0.0/0": [
            {
                "prefix": "0.0.0.0/0",
                "prefixLen": 0,
                "protocol": "ospf",
                "distance": 20,
                "nexthops": [
                    {
                        "ip": "10.1.2.2",
                        "interfaceName": "r3-eth0",
                    }
                ],
            }
        ]
    }
    test_func = partial(
        topotest.router_json_cmp, r3, "show ip route 0.0.0.0/0 json", input_dict
    )
    _, result = topotest.run_and_expect(test_func, None, count=60, wait=1)
    assertmsg = "0.0.0.0/0 from router r2 not installed on r3"
    assert result is None, assertmsg


def test_memory_leak():
    "Run the memory leak test and report results."
    tgen = get_topogen()
    if not tgen.is_memleak_enabled():
        pytest.skip("Memory leak test/report is disabled")

    tgen.report_memory_leaks()


if __name__ == "__main__":
    args = ["-s"] + sys.argv[1:]
    sys.exit(pytest.main(args))
